/* ==========================================================================*/
/* - - - - - - - - - - -  A m b e 2 0 2 0 . C   - - - - - - - - - - - - - -  */
/* ==========================================================================*/
/*	Created   2007-03-10   M. Wheatley                                       */
/*	Modified  2007-03-16   MW                                                */
/*.........................................................................  */
/* VHF Digital Voice Xceiver AMBE 2020 vocoder handler                       */

/*---------------------------------------------------------------------------*/
/*------------------------> I N C L U D E S <--------------------------------*/
/*---------------------------------------------------------------------------*/
#include "Board.h"
#include "irq.h"
#include "ambe2020.h"
#include "usart.h"
#include "codec.h"
#include "Rfcontrol.h"
#include "fpcom.h"
#include <string.h>

 
/*---------------------------------------------------------------------------*/
/*--------------------> L O C A L   D E F I N E S <--------------------------*/
/*---------------------------------------------------------------------------*/
#define AMBE2020_TASK_PRIORITY ( tskIDLE_PRIORITY + 7 )
#define QUEUE_SIZE 3

#define BIT_ERROR_LIMIT 3

#define TONESTATE_OFF 0
#define TONESTATE_PRE 1
#define TONESTATE_POST 2

#define ECHO_MINLENGTH 300

/*---------------------------------------------------------------------------*/
/*-------------------> M E M O R Y  A L L O C A T I O N <--------------------*/
/*---------------------------------------------------------------------------*/
volatile U32BIT XferState;
volatile U32BIT SSCTransferIndex;
tInFrame InFrame;
tOutFrame OutFrame;
U32BIT SSCTransferSize;
S32BIT CurrentVolume;
xSemaphoreHandle xAmbeBlockRdySemaphore;

U32BIT EchoBuffer[ECHO_BUF_LENGTH+3];
S32BIT EchoPointer;
S32BIT EchoLength;

U32BIT ToneState;
U32BIT ToneCount;



/*---------------------------------------------------------------------------*/
/*--------------------------> P R O T O T Y P E S <--------------------------*/
/*---------------------------------------------------------------------------*/
static void Ambe2020Task( void *pvParmeters );

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                  -------------------------                                */
/*                 | A m b e 2 0 2 0 I n i t |                               */
/*                  -------------------------                                */
/*    Called to initialize the SSI port and AMBE2020.                        */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void Ambe2020Init(void )
{
	vSemaphoreCreateBinary( xAmbeBlockRdySemaphore );
	//* Configure PIO controllers to periph mode for TWI (I2C) and SSC
 	AT91F_PIO_CfgOutput( AT91C_BASE_PIOA, SSC_TF ) ;
	AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, SSC_TF ) ;
 	AT91F_PIO_CfgPeriph( AT91C_BASE_PIOA,
  		((unsigned int) SSC_TF   ) |
 		((unsigned int) SSC_TK   ) |
 		((unsigned int) SSC_TD   ) |
 		((unsigned int) SSC_RD   ),// Peripheral A
 		0); // Peripheral B
	//Setup AT91SAM7X Sync Serial Controller
	AT91F_PMC_EnablePeriphClock( AT91C_BASE_PMC, (1<<AT91C_ID_SSC) );	//enable clk
	*AT91C_SSC_CR = AT91C_SSC_SWRST;
	*AT91C_SSC_CMR = 24;	//~1MHz clk
	*AT91C_SSC_RCMR = (1<<16)|AT91C_SSC_CKS_TK|AT91C_SSC_CKO_CONTINOUS|AT91C_SSC_START_TX;
	*AT91C_SSC_RFMR = (AT91C_SSC_DATLEN & (16-1)) | AT91C_SSC_MSBF | AT91C_SSC_FSOS_HIGH;
	
	*AT91C_SSC_TCMR = (1<<16)|(50<<24)|AT91C_SSC_CKS_DIV|AT91C_SSC_CKO_CONTINOUS|AT91C_SSC_CKI|AT91C_SSC_START_RISE_RF;
	*AT91C_SSC_TFMR = (AT91C_SSC_DATLEN & (16-1)) | AT91C_SSC_MSBF | AT91C_SSC_FSOS_HIGH;
	*AT91C_SSC_CR = AT91C_SSC_RXEN|AT91C_SSC_TXEN;
    //* Config SSC interrupt
	AT91F_AIC_ConfigureIt ( AT91C_BASE_AIC, AT91C_ID_SSC, SSC_INTERRUPT_LEVEL,
						AT91C_AIC_SRCTYPE_INT_LEVEL_SENSITIVE, SSCIrqService);
	AT91F_AIC_EnableIt (AT91C_BASE_AIC, AT91C_ID_SSC);	//enable SSC DMA irq service

	xTaskCreate( Ambe2020Task, ( const signed portCHAR * const ) "AMBE", 
				configMINIMAL_STACK_SIZE, NULL, AMBE2020_TASK_PRIORITY, ( xTaskHandle * ) NULL );
	memset(InFrame.All, 0, sizeof(InFrame) );
	InFrame.fld.Hdr = AMBE2020_FRAME_SYNC;
	InFrame.fld.CW1 = 0x0000;

	InFrame.fld.Rate0 = 0x1030;	//FEC block codes
	InFrame.fld.Rate1= 0x4000;
	InFrame.fld.Rate2 = 0x0000;
	InFrame.fld.Rate3 = 0x0000;
	InFrame.fld.Rate4 = 0x0048;
	
	InFrame.fld.DTMFCtrl = 0x00FF;
//	InFrame.fld.CW2 = 0x04;		//Encoder(RF Tx mode) enable
	InFrame.fld.CW2 = 0x00;		//Decoder(RF Rx mode) enable
	
	ToneState = TONESTATE_OFF;
}


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -------------------------                            */
/*                     | A m b e 2 0 2 0 T a s k |                           */
/*                      -------------------------                            */
/*    Task that manages the AMBE2020 chip                                    */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void Ambe2020Task( void *pvParmeters )
{
int i;
	while( (AT91F_PIO_GetInput(AT91C_BASE_PIOA) & EPR_) != 0 )
	{	//wait for chip to ask for first data packet
		vTaskDelay( 0 );
	}
AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, TP5 ) ;
	XferState = XFERSTATE_LOOKING_FOR_SYNC;
	*AT91C_SSC_TFMR = (AT91C_SSC_DATLEN & (16-1)) | AT91C_SSC_MSBF | AT91C_SSC_FSOS_POSITIVE;
	SSCTransferIndex = 0;
	*AT91C_SSC_THR = 0;
	*AT91C_SSC_IER = AT91C_SSC_RXRDY; 
	
	EchoPointer = 0;
	EchoLength = 0;
	ToneCount = 0;
	RxQueueEmpty = TRUE;
	RFRxQHead = 0;
	RFRxQTail = 0;
	for( ;; )	//loop forever in this task
    {
		if( xSemaphoreTake( xAmbeBlockRdySemaphore, ( portTickType )500 ) == pdTRUE )
		{	//block ready from/to ambe
			if( TXSTATE_ON == TxState )
			{	//need to put data from ambe into dstar VDframe and put in tx fifo
				ToneState = TONESTATE_OFF;
				SquelchOpen = FALSE;
				InFrame.fld.CW2 = 0x04;		//Encoder(RF Tx mode) enable
				InFrame.fld.DTMFCtrl = 0x00FF;
				InFrame.fld.CW1 = 0x0000;
				if( FRAMESYNC_SPACING <= TxSyncFrameCount)
				{
					PutDstarVSFrame();
					TxSyncFrameCount = 1;
				}
				else
				{
					PutDstarVDFrame();
					TxSyncFrameCount++;
				}
				RxQueueEmpty = TRUE;
			}
			else if( TXSTATE_OFF == TxState )
			{	//receive mode
				switch(ToneState)
				{
					case TONESTATE_OFF:
						InFrame.fld.DTMFCtrl = 0x00FF;
						if(!RxQueueEmpty)
						{
							if(OutFrame.fld.CW2 == 0x00)
								InFrame.fld.CW2 = 0x03;		//rate is setup so allow Bit error detection
							InFrame.fld.CW1 = 0x0000;
					 		if( OutFrame.fld.BitErrors < BIT_ERROR_LIMIT )	//if is signal present
					 		{
//			 					if(!SquelchOpen)
//			 					{
//									InFrame.fld.DTMFCtrl = 0xF616;
//									ToneState = TONESTATE_PRE;
//									ToneCount = 1;
//			 					}
				 				SquelchOpen = TRUE;
				 				if(	DisplayMode != ECHO_DISPLAY )
									InFrame.fld.Volume = CurrentVolume;		//Decoder volume enable
				 			}
			 				else
			 				{	//no signal present
			 					if(SquelchOpen)
			 					{
									InFrame.fld.DTMFCtrl = 0xF632;
									ToneState = TONESTATE_POST;
									ToneCount = 1;
									InFrame.fld.Volume = CurrentVolume;		//Decoder volume enable
			 					}
			 					else
			 					{
									InFrame.fld.Volume = 0x00;	//Decoder volume disable
			 					}
								SquelchOpen = FALSE;
			 				}
							GetDstarVDFrame();
							if(RFRxQHead == RFRxQTail)
							{
								RxQueueEmpty = TRUE;
								RFRxQHead = 0;
								RFRxQTail = 0;
							}
						}
						else
						{
							InFrame.fld.CW1 = 0x8000;		//flag missing frame
							InFrame.fld.Volume = 0x00;		//Decoder volume disable
							if(RFRxQHead > 1 )
								RxQueueEmpty = FALSE;
						}
						break;
					case TONESTATE_PRE:
						InFrame.fld.DTMFCtrl = 0x00FF;
						ToneState = TONESTATE_OFF;
						GetDstarVDFrame();
						break;
					case TONESTATE_POST:
						InFrame.fld.DTMFCtrl = 0x00FF;
						ToneState = TONESTATE_OFF;
						if(EchoPointer > ECHO_MINLENGTH )
							EchoLength = EchoPointer;
						else
							EchoPointer = 0;
						break;
				}
			}
		}
 	}
}

