/* ==========================================================================*/
/* - - - - - - - - - - - - -  C o d e c . c  - - - - - - - - - - - - - - - - */
/* ==========================================================================*/
/*	Created   2007-03-10   M. Wheatley                                       */
/*	Modified  2007-03-10   MW                                                */
/*.........................................................................  */
/* TLV320AIC23 Audio CODEC Management                                        */

/*---------------------------------------------------------------------------*/
/*------------------------> I N C L U D E S <--------------------------------*/
/*---------------------------------------------------------------------------*/
#include "codec.h"
#include "Board.h"
#include "Irq.h"
#include "TLV320AIC23b.h"

/*---------------------------------------------------------------------------*/
/*--------------------> L O C A L   D E F I N E S <--------------------------*/
/*---------------------------------------------------------------------------*/
#define	FRAM_SLAVE_ADR 0x50


/*---------------------------------------------------------------------------*/
/*------------------> L O C A L    P R O T O T Y P E S <---------------------*/
/*---------------------------------------------------------------------------*/
static void WriteCodec( U32BIT Reg, U32BIT Data );

/*---------------------------------------------------------------------------*/
/*-------------------> M E M O R Y  A L L O C A T I O N <--------------------*/
/*---------------------------------------------------------------------------*/
static BOOL Tog;

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -------------------                                  */
/*                     | C o d e c I n i t |                                 */
/*                      -------------------                                  */
/*    Called to initialize the Codec SSI and control I2C port.               */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void CodecInit ( void )
{
int i;
	Tog = FALSE;
 	//* Configure PIO controllers to periph mode for TWI (I2C)
 	AT91F_PIO_CfgPeriph( AT91C_BASE_PIOA,
 		((unsigned int) SDA    ) |
 		((unsigned int) SCL   ),// Peripheral A
 		0); // Peripheral B

	//Set up I2C Port.
	AT91F_PMC_EnablePeriphClock( AT91C_BASE_PMC, 1 << AT91C_ID_TWI);
	*AT91C_TWI_CR = AT91C_TWI_MSEN;
	*AT91C_TWI_CWGR = (58<<8) + 58;	// CHDIV = MCLK/(400000*2) - 3

	//Setup TLV320AIC23B Codec registers
	WriteCodec(TLV_REG_RESET, TLV_RESETCMD);
	WriteCodec(TLV_REG_POWERDOWN, 0);
  	WriteCodec(TLV_REG_DIGITAL_FORMAT, TLV_MASTERMODE|TLV_INPUT_BITS_32|TLV_DATA_FORMAT_DSP|TLV_DACLRPHZ);
	WriteCodec(TLV_REG_DIGITAL_PATH, 0);

	//Pick Input source
	WriteCodec(TLV_REG_ANALOG_PATH, TLV_DAC_SELECTED|TLV_INPUTSEL_MIC|TLV_MIC_BOOST);

	//Pick Sample Rate
	WriteCodec(TLV_REG_SAMPLE_RATE, TLV_CLKIN_MCLK|TLV_CLKOUT_MCLK|TLV_NORMALRATE|TLV_OVERSAMPRATE_250_256|TLV_RATE_3);	//8KHz

	WriteCodec(TLV_REG_DIGITAL_INTERFACE, TLV_INTERFACE_ACTIVE);

	SetVolume(99);

}


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      ---------------------                                */
/*                     | W r i t e C o d e c |                               */
/*                      ---------------------                                */
/*    Called to Set data to a codec register.                                */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void WriteCodec( U32BIT Reg, U32BIT Data )
{
U32BIT x = *AT91C_TWI_RHR;		//dummy read according to SAM7 errata
	*AT91C_TWI_MMR = (CODEC_SLAVE_ADR<<16)|AT91C_TWI_IADRSZ_1_BYTE;
	x = Reg	| Data;		//combine into 16 bit control and data word
	*AT91C_TWI_IADR = (x>>8) & 0xFF;
	*AT91C_TWI_THR = x & 0xFF;
	*AT91C_TWI_CR = AT91C_TWI_STOP|AT91C_TWI_START;	//START/STOP is automatic?
	while( !(*AT91C_TWI_SR & AT91C_TWI_TXCOMP) ){}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -------------------                                  */
/*                     | S e t V o l u m e |                                 */
/*                     --------------------                                  */
/*    Called to Set the Volume.  Vol range is 0 to 99.                       */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void SetVolume( U8BIT vol )
{
U32BIT x;
//	x = ((U32BIT)vol*100)/154 + 63;
	WriteCodec(TLV_REG_L_HEADPHONE_VC, TLV_HEAD_VOL_MASK );
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -------------------                                  */
/*                     | W r i t e F R A M |                                 */
/*                      -------------------                                  */
/*    Called to Write a Byte to the FRAM at Adr.                             */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void WriteFRAM( U16BIT Adr, U8BIT Data )
{
U32BIT status;
//	status = *AT91C_TWI_RHR;		//dummy read according to SAM7 errata
	*AT91C_TWI_IADR = Adr;
	*AT91C_TWI_MMR = (FRAM_SLAVE_ADR<<16)|AT91C_TWI_IADRSZ_2_BYTE;
	
	*AT91C_TWI_CR = AT91C_TWI_START|AT91C_TWI_MSEN|AT91C_TWI_STOP;
	*AT91C_TWI_THR = Data;
	while( !(*AT91C_TWI_SR & AT91C_TWI_TXCOMP) ){}
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -----------------                                    */
/*                     | R e a d F R A M |                                   */
/*                      -----------------                                    */
/*    Called to Read a Byte from the FRAM at Adr.                            */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
U8BIT ReadFRAM( U16BIT Adr )
{
U32BIT status;
//	status = *AT91C_TWI_RHR;		//dummy read according to SAM7 errata
	*AT91C_TWI_IADR = Adr;
	*AT91C_TWI_MMR = (FRAM_SLAVE_ADR<<16)|AT91C_TWI_IADRSZ_2_BYTE|AT91C_TWI_MREAD;
	*AT91C_TWI_CR = AT91C_TWI_START|AT91C_TWI_MSEN|AT91C_TWI_STOP;
	while( !(*AT91C_TWI_SR & AT91C_TWI_TXCOMP) ){}
	return  (U8BIT)*AT91C_TWI_RHR;
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -----------------------                              */
/*                     | W r i t e 3 2 F R A M |                             */
/*                      -----------------------                              */
/*    Called to Write a 32bit word to the FRAM at byte Adr.                  */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void Write32FRAM( U16BIT Adr, U32BIT Data )
{
t32BITS tmp;
	tmp.all = Data;
	WriteFRAM( Adr, tmp.bytes.b0);
	WriteFRAM( Adr+1, tmp.bytes.b1);
	WriteFRAM( Adr+2, tmp.bytes.b2);
	WriteFRAM( Adr+3, tmp.bytes.b3);
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      ---------------------                                */
/*                     | R e a d 3 2 F R A M |                               */
/*                      ---------------------                                */
/*    Called to Read a 32 bit word from the FRAM at byte Adr.                */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
U32BIT Read32FRAM( U16BIT Adr )
{
t32BITS tmp;
	tmp.bytes.b0 = ReadFRAM( Adr);
	tmp.bytes.b1 = ReadFRAM( Adr+1);
	tmp.bytes.b2 = ReadFRAM( Adr+2);
	tmp.bytes.b3 = ReadFRAM( Adr+3);
	return tmp.all;
}

