/* ==========================================================================*/
/* - - - - - - - - - - - - - -  F P C o m . C   - - - - - - - - - - - - - -  */
/* ==========================================================================*/
/*	Created   2007-03-10   M. Wheatley                                       */
/*	Modified  2007-03-10   MW                                                */
/*.........................................................................  */
/* VHF Digital Voice Xceiver Front Panel Com interface                       */

/*---------------------------------------------------------------------------*/
/*------------------------> I N C L U D E S <--------------------------------*/
/*---------------------------------------------------------------------------*/
#include "Common.h"
#include "Board.h"
#include "fpcom.h"
#include "usart.h"
#include "codec.h"
#include "rfcontrol.h"
#include "irq.h"
#include "ambe2020.h"

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"

/*---------------------------------------------------------------------------*/
/*--------------------> L O C A L   D E F I N E S <--------------------------*/
/*---------------------------------------------------------------------------*/
#define COM_TASK_PRIORITY ( tskIDLE_PRIORITY + 1 )
#define LCD_CLEAR 0x01
#define LCD_LINE1 0x80
#define LCD_LINE2 0xC0
#define LCD_CHAR_LINE 16
#define LCD_CURSORON 0x0D
#define LCD_CURSOROFF 0x0C

#define SW_FASTDOWNLIMIT 3
#define SW_SLOWDOWNLIMIT 50

#define MAX_FREQ_CHNG_POS 4

/*---------------------------------------------------------------------------*/
/*------------------------> M A C R O S <------------------------------------*/
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/*-------------------> M E M O R Y  A L L O C A T I O N <--------------------*/
/*---------------------------------------------------------------------------*/
const U32BIT DUMBASS_TBL[16] = 
{	//conversion of flipped bits on LCD data bus
	(0x0<<24),(0x8<<24),(0x4<<24),(0xC<<24),(0x2<<24),(0xA<<24),(0x6<<24),(0xE<<24),
	(0x1<<24),(0x9<<24),(0x5<<24),(0xD<<24),(0x3<<24),(0xB<<24),(0x7<<24),(0xF<<24)
};

const U32BIT DIGIT_POS_TBL[MAX_FREQ_CHNG_POS+1] = 
{
	9,8,7,6,4
};

const S32BIT DIGIT_MULT_TBL[MAX_FREQ_CHNG_POS+1] = 
{
	100,1000,10000,100000,1000000
};

const S32BIT ROTARY_TBL[16] = // last AB , present AB
{							// AB AB
	0,						// 00 00
	-1,						// 00 01
	1,						// 00 10
	0,						// 00 11
	0,						// 01 00
	0,						// 01 01
	0,						// 01 10
	0,						// 01 11
	0,						// 10 00
	0,						// 10 01
	0,						// 10 10
	0,						// 10 11
	0,						// 11 00
	0,						// 11 01
	0,						// 11 10
	0						// 11 11
};


U32BIT DisplayMode;
BOOL FreqChangeModeActive;
S32BIT ChangeDigitPos;
static U32BIT CursorPos;
static char Buf[80];
volatile S32BIT RotaryChange;
static U32BIT DisplayTimer;
static U32BIT LastRotSwState;
static U32BIT PresentRotSwState;
static U32BIT SWDownTimer;
static U32BIT VolDisplayTimer;
static U32BIT LastDisplayMode;


/*---------------------------------------------------------------------------*/
/*--------------------------> P R O T O T Y P E S <--------------------------*/
/*---------------------------------------------------------------------------*/
void FPComTask( void *pvParmeters );
void InitLCD();
void WriteLCDdata(U8BIT data);
void WriteLCDcmd(U8BIT cmd);
void LCDWriteStr(U8BIT x, char* str);
void LCDWriteDec8(U8BIT x, U8BIT val);
void LCDClear();
void LCDSetCursor(U8BIT pos);
static void ItoA8( char* str, U8BIT val);
static void XtoA( char* str, int val);
static void ServiceRotSwitch(void);
static void DisplayFreq(U32BIT Freq);
static void EchoDisplay();
static void TuneDisplay();
static void NormalDisplay();
static void FreqDigitChange(BOOL Up);


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -------------------                                  */
/*                     | F P C o m I n i t |                                 */
/*                      -------------------                                  */
/*    Called to initialize the FPanel Communications interface.              */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void FPComInit( void )
{
	//Init LCD control lines
	AT91F_PIO_CfgOutput( AT91C_BASE_PIOA, LCD_RS|LCD_D7|LCD_D6|LCD_D5|LCD_D4|LCD_E|LCD_BK ) ;
	AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LCD_RS|LCD_D7|LCD_D6|LCD_D5|LCD_D4|LCD_E|LCD_BK ) ;
	AT91F_PIO_SetOutput( AT91C_BASE_PIOA, LCD_BK ) ;
	*AT91C_PIOA_IFER = (UPSW_|DNSW_|MODESW_);	//enable glitch filters
	AT91F_AIC_ConfigureIt ( AT91C_BASE_AIC, AT91C_ID_PIOA, PIOA_INTERRUPT_LEVEL,
							AT91C_AIC_SRCTYPE_INT_LEVEL_SENSITIVE, PIOAIrqService);
	AT91F_AIC_EnableIt (AT91C_BASE_AIC, AT91C_ID_PIOA);
	*AT91C_PIOA_IER = (UPSW_|DNSW_);	//enable change interrupts
	xTaskCreate( FPComTask, ( const signed portCHAR * const ) "COM", 
					configMINIMAL_STACK_SIZE, NULL, COM_TASK_PRIORITY, ( xTaskHandle * ) NULL );
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -------------------                                  */
/*                     | F P C o m T a s k |                                 */
/*                      -------------------                                  */
/*    Task that manages sending and receiving  messages over the I2C bus     */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void FPComTask( void *pvParmeters )
{
int i;
	InitLCD();
	vTaskDelay( 500);
	LCDWriteStr(0, " MoeTronix DVX ");
	vTaskDelay( 2000);
	LCDClear();
	DisplayTimer = 0;
	RotaryChange = 0;
	SWDownTimer = 0;
	EchoLength = 0;
	VolDisplayTimer = 5;
	DisplayMode = NORMAL_DISPLAY;
	LastDisplayMode = ~NORMAL_DISPLAY;
	FreqChangeModeActive = FALSE;
	ChangeDigitPos = -1;
    for( ;; )	//loop forever in this task
	{
		vTaskDelay( 20 );
		ServiceRotSwitch();
		switch(DisplayMode)
		{
			case NORMAL_DISPLAY:
				NormalDisplay();
				break;
			case ECHO_DISPLAY:
				EchoDisplay();
				break;
			case TUNE_DISPLAY:
				TuneDisplay();
				break;
			default:
				DisplayMode = NORMAL_DISPLAY;
				NormalDisplay();
				break;
		}
		LastDisplayMode = DisplayMode;
		if( (AT91F_PIO_GetInput(AT91C_BASE_PIOA) & MODESW_) == 0 )
		{
			if(SWDownTimer < SW_SLOWDOWNLIMIT)
			{
				SWDownTimer++;
			}
			else
			{
				AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LCD_BK ) ;
				if(	ChangeDigitPos >= 0 )
				{
					ChangeDigitPos = -1;
					WriteLCDcmd(LCD_CURSOROFF);
				}
			}
		}
		else
		{
			if(SquelchOpen && ( ECHO_DISPLAY == DisplayMode ) )
			{
				i++;
				if( (i & 0x07) == 0)
				{	//Blink LCD backlight when recording echo test
					if(AT91F_PIO_GetInput(AT91C_BASE_PIOA) & LCD_BK )
						AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LCD_BK );
					else
						AT91F_PIO_SetOutput( AT91C_BASE_PIOA, LCD_BK );
				}
			}
			else
			{
				AT91F_PIO_SetOutput( AT91C_BASE_PIOA, LCD_BK ) ;
			}
			if (SWDownTimer >= SW_SLOWDOWNLIMIT )
			{
				if(!FreqChangeModeActive)
				{
					DisplayMode++;
					if(DisplayMode > ECHO_DISPLAY)
						DisplayMode = NORMAL_DISPLAY;
				}
				else
				{
					FreqChangeModeActive = FALSE;
					ChangeDigitPos = -1;
					WriteLCDcmd(LCD_CURSOROFF);
				}
			}
			else if(SWDownTimer >= SW_FASTDOWNLIMIT)
			{
				if(!FreqChangeModeActive)
				{
					FreqChangeModeActive = TRUE;
					ChangeDigitPos = 0;
					LCDSetCursor(DIGIT_POS_TBL[0]);
					WriteLCDcmd(LCD_CURSORON);
				}
				else
				{
					ChangeDigitPos++;
					if(	ChangeDigitPos > MAX_FREQ_CHNG_POS)
					{
						ChangeDigitPos = 0;
					}
					LCDSetCursor(DIGIT_POS_TBL[ChangeDigitPos]);
				}
			}
			SWDownTimer = 0;
		}
	}
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                       ---------------                                     */
/*                      | I n i t L C D |                                    */
/*                       ---------------                                     */
/*    Initialize LCD to 4 bit mode and reset display                         */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void InitLCD()
{
	AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LCD_E|LCD_RS ) ;
	vTaskDelay( 20 );
	AT91F_PIO_SetOutput( AT91C_BASE_PIOA, LCD_E ) ;
	vTaskDelay( 0 );
	AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LCD_DATAMASK ) ;
	AT91F_PIO_SetOutput( AT91C_BASE_PIOA, DUMBASS_TBL[0x03] ) ;
	AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LCD_E ) ;

	vTaskDelay( 10 );
	AT91F_PIO_SetOutput( AT91C_BASE_PIOA, LCD_E ) ;
	AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LCD_DATAMASK ) ;
	AT91F_PIO_SetOutput( AT91C_BASE_PIOA, DUMBASS_TBL[0x03] ) ;
	vTaskDelay( 0 );
	AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LCD_E ) ;

	vTaskDelay( 2 );
	AT91F_PIO_SetOutput( AT91C_BASE_PIOA, LCD_E ) ;
	AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LCD_DATAMASK ) ;
	AT91F_PIO_SetOutput( AT91C_BASE_PIOA, DUMBASS_TBL[0x03] ) ;
	vTaskDelay( 0 );
	AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LCD_E ) ;
	
	vTaskDelay( 1 );
	AT91F_PIO_SetOutput( AT91C_BASE_PIOA, LCD_E ) ;
	AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LCD_DATAMASK ) ;
	AT91F_PIO_SetOutput( AT91C_BASE_PIOA, DUMBASS_TBL[0x02] ) ;
	vTaskDelay( 0 );
	AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LCD_E ) ;
	
	WriteLCDcmd(0x20);		//now can use regular writing routines
	WriteLCDcmd(0x08);
	WriteLCDcmd(LCD_CLEAR);
	WriteLCDcmd(0x06);
	WriteLCDcmd(0x0E);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                       ---------------------------                         */
/*                      | N o r m a l D i s p l a y |                        */
/*                       ---------------------------                         */
/*    Managages the normal xceiver display mode                              */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void NormalDisplay()
{
BOOL First = (DisplayMode != LastDisplayMode);
	if(FreqChangeModeActive)
		return;
	if( (AT91F_PIO_GetInput(AT91C_BASE_PIOA) & PTT_) == 0 )
	{
		if( TXSTATE_OFF == TxState )
		{
			LCDClear();
			LCDWriteStr(0, "Tx");
			StartTx();
		}
		if(++DisplayTimer > 25)
		{	//update display info
			DisplayTimer = 0;
			DisplayFreq(CurrentTxFreq);
		}
	}
	else
	{
		if( TXSTATE_ON == TxState )
		{
			EndTx();
		}
		else if( First || (TXSTATE_DONE == TxState) )
		{
			LCDClear();
			LCDWriteStr(0, "Rx");
			StartRx();
		}
		if(VolDisplayTimer)
		{
			LCDWriteStr(10, " V");
			LCDWriteDec8(12, (U8BIT)CurrentVolume);
			LCDWriteStr(14, "  ");
			VolDisplayTimer--;
		}
		else
		{
			if(++DisplayTimer > 25)
			{	//update display info
				DisplayTimer = 0;
				DisplayFreq(CurrentRxFreq);
				LCDWriteStr(10, "-");
				LCDWriteDec8(11, (U8BIT)(-RxdBm));
				LCDWriteStr(14, "dB");
			}
		}
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                       -----------------------                             */
/*                      | E c h o D i s p l a y |                            */
/*                       -----------------------                             */
/*    Managages the echo mode xceiver display                                */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void EchoDisplay()
{
BOOL First = (DisplayMode != LastDisplayMode);
	if( !SquelchOpen && (EchoLength != 0 ) )
	{
		if( TXSTATE_OFF == TxState )
		{
			LCDClear();
			LCDWriteStr(0, "ET");
			StartTx();
		}
		if(++DisplayTimer > 25)
		{	//update display info
			DisplayTimer = 0;
			DisplayFreq(CurrentTxFreq);
		}
	}
	else
	{
		if( TXSTATE_ON == TxState )
		{
			EndTx();
		}
		else if( First || (TXSTATE_DONE == TxState) )
		{
			LCDClear();
			LCDWriteStr(0, "ER");
			StartRx();
		}
		if(++DisplayTimer > 25)
		{	//update display info
			DisplayTimer = 0;
			DisplayFreq(CurrentRxFreq);
			LCDWriteStr(10, "-");
			LCDWriteDec8(11, (U8BIT)(-RxdBm));
			LCDWriteStr(14, "dB");
		}
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                       -----------------------                             */
/*                      | T u n e D i s p l a y |                            */
/*                       -----------------------                             */
/*    Managages the tune mode xceiver display                                */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void TuneDisplay()
{
BOOL First = (DisplayMode != LastDisplayMode);
	if( (AT91F_PIO_GetInput(AT91C_BASE_PIOA) & PTT_) == 0 )
	{
		if( TXSTATE_OFF == TxState )
		{
			LCDClear();
			LCDWriteStr(0, "CW");
			StartTx();
		}
		if(++DisplayTimer > 25)
		{	//update display info
			DisplayTimer = 0;
			DisplayFreq(CurrentTxFreq);
		}
	}
	else
	{
		if( TXSTATE_ON == TxState )
		{
			EndTx();
		}
		else if( First || (TXSTATE_DONE == TxState) )
		{
			LCDClear();
			LCDWriteStr(0, "CW");
			StartRx();
		}
		if(++DisplayTimer > 25)
		{	//update display info
			DisplayTimer = 0;
			DisplayFreq(CurrentRxFreq);
			LCDWriteStr(10, "-");
			LCDWriteDec8(11, (U8BIT)(-RxdBm));
			LCDWriteStr(14, "dB");
		}
	}
}


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                   -------------------------------                         */
/*                  | F r e q D i g i t C h a n g e |                        */
/*                   -------------------------------                         */
/*    Called to inc or dec te current digit position                         */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void FreqDigitChange(BOOL Up)
{
S32BIT Change;
	if(ChangeDigitPos < 0)
		return;
	Change = DIGIT_MULT_TBL[ChangeDigitPos];
	if(!Up)
		Change = -Change;
	CurrentRxFreq = CurrentRxFreq + Change;
	if(CurrentRxFreq > 148000000)
		CurrentRxFreq = 148000000;
	if(CurrentRxFreq < 144000000)
		CurrentRxFreq = 144000000;
	CurrentTxFreq = CurrentRxFreq;		//simplex only for now
	SetTxFrequency(CurrentTxFreq);
	Write32FRAM( EEADR_TXFREQ, CurrentTxFreq);
	SetRxFrequency(CurrentRxFreq);
	Write32FRAM( EEADR_RXFREQ, CurrentRxFreq);
	LCDSetCursor(2);
	DisplayFreq(CurrentRxFreq);
	LCDSetCursor(DIGIT_POS_TBL[ChangeDigitPos]);
}


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                       ---------------------------                         */
/*                      | L C D D e b u g 3 2 B i t |                        */
/*                       ---------------------------                         */
/*    Writes a 32 bit value in hex to LCD                                    */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void LCDDebug32Bit(unsigned int val)
{
	LCDClear();
	XtoA( Buf, val);
	LCDWriteStr( 0, Buf);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                     -------------------------                             */
/*                    | L C D S e t C u r s o r |                            */
/*                     -------------------------                             */
/*    Sets LCD edit cursor to position x (0 to 15)                           */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void LCDSetCursor(U8BIT x)
{
	if(	x >= 8 )
		WriteLCDcmd(0x80 + 0x40 + x - 8);
	else
		WriteLCDcmd(0x80 + x);
	CursorPos = x;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                       -----------------------                             */
/*                      | L C D W r i t e S t r |                            */
/*                       -----------------------                             */
/*    Writes a string at position x (0 to 15)                                */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void LCDWriteStr(U8BIT x, char* str)
{
int i;
	i = 0;
	if(	x >= 8 )
		WriteLCDcmd(0x80 + 0x40 + x - 8);
	else
		WriteLCDcmd(0x80 + x);
	CursorPos = x;
	while( str[i] && (CursorPos<16 ) )
	{
		WriteLCDdata(str[i++]);
	};
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                       ---------------------------                         */
/*                      | L C D W r i t e D e c l 8 |                        */
/*                       ---------------------------                         */
/*    Writes a 3 place decimal value  at position x (0 to 15)                */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void LCDWriteDec8(U8BIT x, U8BIT val)
{
	ItoA8( Buf, val);
	LCDWriteStr( x, Buf);
}



/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                       -----------------                                   */
/*                      | L C D C l e a r |                                  */
/*                       -----------------                                   */
/*    Clears LCD line                                                        */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void LCDClear()
{
	WriteLCDcmd(LCD_CLEAR);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                       -----------------------                             */
/*                      | W r i t e L C D c m d |                            */
/*                       -----------------------                             */
/*    Writes a low level command byte to the LCD                             */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void WriteLCDcmd(U8BIT cmd)
{
	AT91F_PIO_SetOutput( AT91C_BASE_PIOA, LCD_E ) ;
	AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LCD_DATAMASK|LCD_RS ) ;
	AT91F_PIO_SetOutput( AT91C_BASE_PIOA, DUMBASS_TBL[cmd>>4] ) ;
	vTaskDelay( 0 );
	AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LCD_E ) ;
	vTaskDelay( 0 );
	
	AT91F_PIO_SetOutput( AT91C_BASE_PIOA, LCD_E ) ;
	AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LCD_DATAMASK ) ;
	AT91F_PIO_SetOutput( AT91C_BASE_PIOA, DUMBASS_TBL[cmd&0xF] ) ;
	vTaskDelay( 0 );
	AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LCD_E ) ;
	vTaskDelay( 3 );
	if(cmd==LCD_CLEAR)
		CursorPos = 0;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                       -------------------------                           */
/*                      | W r i t e L C D d a t a |                          */
/*                       -------------------------                           */
/*    Writes a low level data byte to the LCD                                */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void WriteLCDdata(U8BIT data)
{
	if(	8 ==CursorPos )
		WriteLCDcmd(0x80 + 0x40);
	
	AT91F_PIO_SetOutput( AT91C_BASE_PIOA, LCD_E|LCD_RS ) ;
	AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LCD_DATAMASK ) ;
	AT91F_PIO_SetOutput( AT91C_BASE_PIOA, DUMBASS_TBL[data>>4] ) ;
	vTaskDelay( 0 );
	AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LCD_E ) ;
	vTaskDelay( 0 );
	
	AT91F_PIO_SetOutput( AT91C_BASE_PIOA, LCD_E ) ;
	AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LCD_DATAMASK ) ;
	AT91F_PIO_SetOutput( AT91C_BASE_PIOA, DUMBASS_TBL[data&0xF] ) ;
	vTaskDelay( 0 );
	AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LCD_E ) ;
	vTaskDelay( 1 );
	CursorPos++;
}


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                           -----------                                     */
/*                          | I t o A 8 |                                    */
/*                           -----------                                     */
/* Create an 8 bit 'val' in decimal to the string 'str'                      */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void ItoA8( char* str, U8BIT val)
{
BOOL Leadzero;
int i,j;
unsigned int x;
unsigned char ch;
	i = 0;
	Leadzero = TRUE;
	ch = val/100; val = val - 100*ch;
	if(ch) Leadzero = FALSE;
	if(!Leadzero) {	ch += '0'; str[i++] = ch; }
	ch = val/10; val = val - 10*ch;
	if(ch) Leadzero = FALSE;
	if(!Leadzero) {	ch += '0'; str[i++] = ch; }
	ch = val;
	ch += '0';
	str[i++] = ch;
	while(i<3)
		str[i++] = ' ';
	str[i] = 0;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                           ---------                                       */
/*                          | X t o A |                                      */
/*                           ---------                                       */
/* Append an integer 'val' in HEX to the string 'str'                        */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void XtoA( char* str, int val)
{
int i,j;
unsigned int x;
unsigned char ch;
	i = 0;
	str[i++] = 'x';
	for(j=0; j<8; j++)
	{
		x = (unsigned int)(val&0xF0000000);
		x >>= 28;
		ch = (char)x;
		if(ch<10)
			ch += '0';
		else
			ch += ('A'-10);
		str[i++] = ch;
		if(i==5)
			str[i++] =' ';
		val = val<<4;
	}
	str[i] = 0;
}
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                   -----------------------                                 */
/*                  | D i s p l a y F r e q |                                */
/*                   -----------------------                                 */
/*    This routine is called to manage the rotary encoder switch.            */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void DisplayFreq(U32BIT Freq)
{
int i;
unsigned char ch;
	i = 0;
	ch = Freq/100000000; Freq = Freq - 100000000*ch;
	ch += '0'; Buf[i++] = ch;	//100M
	ch = Freq/10000000; Freq = Freq - 10000000*ch;
	ch += '0'; Buf[i++] = ch;	//10M
	ch = Freq/1000000; Freq = Freq - 1000000*ch;
	ch += '0'; Buf[i++] = ch;	//1M
	Buf[i++] = '.';
	ch = Freq/100000; Freq = Freq - 100000*ch;
	ch += '0'; Buf[i++] = ch;	//100K
	ch = Freq/10000; Freq = Freq - 10000*ch;
	ch += '0'; Buf[i++] = ch;	//10K
	ch = Freq/1000; Freq = Freq - 1000*ch;
	ch += '0'; Buf[i++] = ch;	//1K
	ch = Freq/100; Freq = Freq - 100*ch;
	ch += '0'; Buf[i++] = ch;	//100
	Buf[i] = 0;
	LCDWriteStr( 2, Buf);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                   ---------------------------------                       */
/*                  | S e r v i c e R o t S w i t c h |                      */
/*                   ---------------------------------                       */
/*    This routine is called to manage the rotary encoder switch.            */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void ServiceRotSwitch(void)
{
	if( 0 != RotaryChange)	//process any rotary encoder events
	{
		if(FreqChangeModeActive && ( AT91F_PIO_GetInput(AT91C_BASE_PIOA) & MODESW_ ) )
		{
			FreqDigitChange( (RotaryChange>0) );
			RotaryChange = 0;
		}
		else
		{
			if(	NORMAL_DISPLAY == DisplayMode )
			{
				if(RotaryChange > 0)
					CurrentVolume += 1;
				else
					CurrentVolume -= 1;
				RotaryChange = 0;
				if( CurrentVolume > 25)
					CurrentVolume = 25;
				if( CurrentVolume < 0)
					CurrentVolume = 0;
				WriteFRAM( EEADR_RXVOL, (U8BIT)CurrentVolume );
				VolDisplayTimer = 50;
			}
		}
	}
}
