/* ==========================================================================*/
/* - - - - - - - - - - -  R F C o n t r o l . C   - - - - - - - - - - - - - -*/
/* ==========================================================================*/
/*	Created   2007-03-10   M. Wheatley                                       */
/*	Modified  2007-03-16   MW                                                */
/*.........................................................................  */
/* VHF Digital Voice Xceiver ADF7021 handler                                 */

/*---------------------------------------------------------------------------*/
/*------------------------> I N C L U D E S <--------------------------------*/
/*---------------------------------------------------------------------------*/
#include "Board.h"
#include "Rfcontrol.h"
#include "irq.h"
#include "fpcom.h"
#include "adf7021.h"
#include "ambe2020.h"
#include "codec.h"
#include "usart.h"
#include  <stdlib.h>
#include <string.h>
 
/*---------------------------------------------------------------------------*/
/*--------------------> L O C A L   D E F I N E S <--------------------------*/
/*---------------------------------------------------------------------------*/

//#define VCO_OFFSET	1332
#define VCO_OFFSET	1700

#define RFCONTROL_TASK_PRIORITY ( tskIDLE_PRIORITY + 6 )
#define QUEUE_SIZE 3

#define FRAMESYNC2 0x16
#define FRAMESYNC1 0x2D
#define FRAMESYNC0 0x55

#define ENDSYNC2 0xF5
#define ENDSYNC1 0x90
#define ENDSYNC0 0xAA

#define SCRAMBLE_TBLSIZE 64	//pwr of 2

#define SQUELCH_LEVEL -90

#define HEADER_LENGTH 26

/*---------------------------------------------------------------------------*/
/*-------------------> M E M O R Y  A L L O C A T I O N <--------------------*/
/*---------------------------------------------------------------------------*/
static tADF721_R0 Reg0;
static tADF721_R1 Reg1;
static tADF721_R2 Reg2;
static tADF721_R3 Reg3;
static tADF721_R4 Reg4;
static tADF721_R5 Reg5;
static tADF721_R6 Reg6;
static tADF721_R7 Reg7;
static tADF721_R8 Reg8;
static tADF721_R9 Reg9;
static tADF721_R10 Reg10;
static tADF721_R11 Reg11;
static tADF721_R12 Reg12;
static tADF721_R13 Reg13;
static tADF721_R14 Reg14;
static tADF721_R15 Reg15;

U32BIT CurrentRxFreq;
U32BIT CurrentTxFreq;

BOOL SquelchOpen;
S16BIT RxdBm;
U32BIT RxState;
U32BIT TxState;
U32BIT ScramblePtr;
tDstarVDFrame RFRxDataQueue[RFRQ_SIZE];
U32BIT RFTxDataQueue[RFTQ_SIZE];
U32BIT RFTxQHead;
volatile U32BIT RFTxQTail;
U32BIT RFRxQTail;
BOOL RxQueueEmpty;
volatile U32BIT RFRxQHead;
volatile U32BIT RxShiftWord;
volatile U32BIT TxShiftWord;
volatile U32BIT TxShiftCount;
volatile U32BIT RxShiftCount;
volatile U32BIT RxWordCount;

U32BIT TxSyncFrameCount;
tDstarVDFrame FirstSyncFrame;
tDstarVDFrame VSFrame;
tDstarVDFrame VDFrame;

const U8BIT SCRAMBLE_TBL[] = 
{
	0x70, 0x4F, 0x93, 0x40, 0x64, 0x74, 0x6D, 0x30, 
	0x2B, 0xE7, 0x2D, 0x54, 0x5F, 0x8A, 0x1D, 0x7F, 
	0xB8, 0xA7, 0x49, 0x20, 0x32, 0xBA, 0x36, 0x98, 
	0x95, 0xF3, 0x16, 0xAA, 0x2F, 0xC5, 0x8E, 0x3F, 
	0xDC, 0xD3, 0x24, 0x10, 0x19, 0x5D, 0x1B, 0xCC, 
	0xCA, 0x79, 0x0B, 0xD5, 0x97, 0x62, 0xC7, 0x1F, 
	0xEE, 0x69, 0x12, 0x88, 0x8C, 0xAE, 0x0D, 0x66, 
	0xE5, 0xBC, 0x85, 0xEA, 0x4B, 0xB1, 0xE3, 0x0F, 
	0xF7, 0x34, 0x09, 0x44, 0x46, 0xD7, 0x06, 0xB3, 
	0x72, 0xDE, 0x42, 0xF5, 0xA5, 0xD8, 0xF1, 0x87, 
	0x7B, 0x9A, 0x04, 0x22 
};



const S16BIT DBM_TBL[16] = 
{
	86,		//0,0,0,0
	0,		//0,0,0,1 invalid
	0,		//0,0,1,0 invalid	
	0,		//0,0,1,1 invalid	
	58,		//0,1,0,0	
	38,		//0,1,0,1
	24,		//0,1,1,0
	0,		//0,1,1,1 invalid	
	0,		//1,0,0,0 invalid	
	0,		//1,0,0,1 invalid	
	0,		//1,0,1,0
	0,		//1,0,1,1 invalid	
	0,		//1,1,0,0 invalid	
	0,		//1,1,0,1 invalid	
	0,		//1,1,1,0 invalid	
	0		//1,1,1,1 invalid	
};

U32BIT DS_HEADER_MSB[24] = {		//sync + 660 bit fixed header for AE4JY
	0x55555555, 0x55555555, 0x55555557, 0x6502f3bf,
	0x9803abdf, 0xb0cc50ed, 0x7a95e879, 0x52ae404c,
	0xcef41409, 0x34c6accc, 0xc44f0a3e, 0xc8cfbc27,
	0x1229c317, 0x88f433c4, 0xbc5d275f, 0x61f517de,
	0x6fbb0b53, 0x16d8575a, 0x8331b8cc, 0x6a5743ce,
	0x2e0c1ddb, 0x380a9622, 0x19f526e7, 0xa66d2774
};

U32BIT DS_HEADER[HEADER_LENGTH] = {		//sync + 660 bit fixed header for AE4JY
	0xAAAAAAAA, 0xAAAAAAAA,
	0xAAAAAAAA, 0xAAAAAAAA, 0xEAAAAAAA, 0xFDCF40A6,
	0xFBD5C019, 0xB70A330D, 0x9E17A95E, 0x3202754A,
	0x90282F73, 0x3335632C, 0x7C50F223, 0xE43DF313,
	0xE8C39448, 0x23CC2F11, 0xFAE4BA3D, 0x7BE8AF86,
	0xCAD0DDF6, 0x5AEA1B68, 0x331D8CC1, 0x73C2EA56,
	0xDBB83074, 0x4469501C, 0xE764AF98, 0x2EE4B665
};

/*---------------------------------------------------------------------------*/
/*--------------------------> P R O T O T Y P E S <--------------------------*/
/*---------------------------------------------------------------------------*/
static void RFControlTask( void *pvParmeters );
static void Write7021(U32BIT val);
static U16BIT Read7021(void);

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                  ---------------------------                              */
/*                 | R F C o n t r o l I n i t |                             */
/*                  ---------------------------                              */
/*    Called to initialize the SPI port and ADF7021.                         */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void RFControlInit(void )
{
	//* enable the clock of the SPI port
	AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1<<AT91C_ID_SPI ) ;
 	//* Configure PIO controllers to periph mode for SPI
 	AT91F_PIO_CfgPeriph( AT91C_BASE_PIOA,
		((unsigned int) SWD  ) |
 		((unsigned int) CS7021_  ) |
 		((unsigned int) MISO7021  ) |
 		((unsigned int) MOSI7021  ) |
 		((unsigned int) SCK7021   ) , // Peripheral A
 		((unsigned int) RXCLK_7021   )	// Peripheral B
 		); 
	*AT91C_SPI_CR = AT91C_SPI_SWRST;
	*AT91C_SPI_CR = AT91C_SPI_SPIEN; // (SPI) Control Register
	*AT91C_SPI_MR = AT91C_SPI_MSTR|AT91C_SPI_PS_VARIABLE|AT91C_SPI_MODFDIS; // (SPI) Mode Register
	AT91C_SPI_CSR[0] = AT91C_SPI_BITS_16|AT91C_SPI_NCPHA|AT91C_SPI_CSAAT|(4<<8); 	//SPI clk == MCK/4
	AT91C_SPI_CSR[1] = AT91C_SPI_BITS_9|AT91C_SPI_CSAAT|(4<<8); 	//9 bit MSBs of readback
	AT91C_SPI_CSR[2] = AT91C_SPI_BITS_8|AT91C_SPI_CSAAT|(4<<8); 	//8 bit LSBs of readback
//Initialize the ADF7021 registers
	Reg0.all = 0; Reg1.all = 0;	Reg2.all = 0; Reg3.all = 0;	Reg4.all = 0;
	Reg5.all = 0; Reg6.all = 0; Reg7.all = 0; Reg8.all = 0; Reg9.all = 0;
	Reg10.all = 0; Reg11.all = 0; Reg12.all = 0; Reg13.all = 0; Reg14.all = 0;
	Reg15.all = 0;

	Reg0.fld.Reg = 0;
	Reg0.fld.Fract = 18208;
	Reg0.fld.Int = 44;
	Reg0.fld.MuxOut = 2;	//PLL Lock output
	Reg0.fld.TR = 1;		//Rx
	Reg0.fld.Uart = 0;
	
	Reg1.fld.Reg = 1;
	Reg1.fld.Rcnt = 3;		//PFD = clk/3
	Reg1.fld.CoutDiv = 0;	//no clock out
	Reg1.fld.X2 = 0;		//no xtal doubler
	Reg1.fld.OscEn = 1;		//Osc on
	Reg1.fld.Xbias = 2;		//30uA xtal bias
	Reg1.fld.Cpi = 2;	
	Reg1.fld.VcoEn = 1;
	Reg1.fld.VcoDiv2 = 1;
	Reg1.fld.VcoBias = 2;
	Reg1.fld.VcoAdj = 0;
	Reg1.fld.VcoLext = 1;
	
	Reg2.fld.Reg = 2;
	Reg2.fld.ModSch = 1;
	Reg2.fld.PaEn = 1;
	Reg2.fld.PwrRamp = 4;
	Reg2.fld.PaBias = 2;
	Reg2.fld.Power = 0x3F;
	Reg2.fld.TxDev = 24;	//24 == +/- 1200
	Reg2.fld.TxInv = 0;
	Reg2.fld.Narrow = 1;
	
	Reg3.fld.Reg = 3;
	Reg3.fld.BClkDiv = 2;	//BBOS clk = xtal/16 = 1.2288 MHz
	Reg3.fld.DemClkDiv = 8;	// Demod clk = xtal/8 = 2.4576 MHz
	Reg3.fld.CDRDiv = 16;	//CDR_CLK = 4800*32 Hz
	Reg3.fld.SeqDiv = 196;	//SEQ_CLK = 100.3KHz
	Reg3.fld.AgcDiv = 10;	//AGC update rate = 10.03 KHz 
	
	Reg4.fld.Reg = 4;
	Reg4.fld.DemSch = 1;	//2FSK Correlator demod
//	Reg4.fld.DemSch = 0;	//2FSK linear demod
 	Reg4.fld.DiscBW = 510;	//K=83
	Reg4.fld.Dot = 1;
	Reg4.fld.PostBW = 9;
	Reg4.fld.IFBW = 0;
	Reg4.fld.RxInv = 0;
	
	Reg5.fld.Reg = 5;
	
	Reg6.fld.Reg = 6;

	Reg7.fld.Reg = 7;
	Reg7.fld.ADCMode = 0;	//A/D is RSSI
	Reg7.fld.ReadSel = 4;	//readback AFC
//	Reg7.fld.ReadSel = 5;	//readback RSSI
	
	Reg8.fld.Reg = 8;

	Reg9.fld.Reg = 9;
	
	Reg10.fld.Reg = 10;
	Reg10.fld.AfcEn = 0;
	Reg10.fld.AfcScale = 427;
	Reg10.fld.AfcRange = 5;//75;
	Reg10.fld.KI = 11;		//2^11 
	Reg10.fld.KP = 4;		//2^4 
	
	Reg11.fld.Reg = 11;
	Reg11.fld.SyncLen = 1;	//16 bits
	Reg11.fld.SyncTol = 0;
	Reg11.fld.SyncWord = 0xB468;	//Voice data frame sync word
	
	Reg12.fld.Reg = 12;
	Reg12.fld.DataLen = 1;
	Reg12.fld.LockPin = 2;

	Reg13.fld.Reg = 13;
	Reg14.fld.Reg = 14;
	Reg15.fld.Reg = 15;
	
    //* setup FIQ interrupt for rx data clock
	AT91F_AIC_ConfigureIt ( AT91C_BASE_AIC, AT91C_ID_FIQ, FIQ_INTERRUPT_LEVEL,AT91C_AIC_SRCTYPE_EXT_POSITIVE_EDGE, FIQIrqService);
	AT91F_AIC_EnableIt (AT91C_BASE_AIC, AT91C_ID_FIQ);
    //* setup INT1 (SWD) interrupt (frame sync)
	AT91F_AIC_ConfigureIt ( AT91C_BASE_AIC, AT91C_ID_IRQ1, IRQ1_INTERRUPT_LEVEL,AT91C_AIC_SRCTYPE_EXT_POSITIVE_EDGE, Irq1Service);
	AT91F_AIC_EnableIt (AT91C_BASE_AIC, AT91C_ID_IRQ1);
	CurrentTxFreq = 0;
	CurrentRxFreq = 0;
	RFTxQHead = 0;
	RFTxQTail = 0;
	RFRxQHead = 0;
	RFRxQTail = 0;
	memset(FirstSyncFrame.Buf32, 0xAA, sizeof(FirstSyncFrame.Buf32) );
	FirstSyncFrame.sync.FrameSync[0] = FRAMESYNC0;
	FirstSyncFrame.sync.FrameSync[1] = FRAMESYNC1;
	FirstSyncFrame.sync.FrameSync[2] = FRAMESYNC2;
	VDFrame.data.UsrData[0] = 0x55;
	VDFrame.data.UsrData[1] = 0x55;
	VDFrame.data.UsrData[2] = 0x55;
	VSFrame.sync.FrameSync[0] = FRAMESYNC0;
	VSFrame.sync.FrameSync[1] = FRAMESYNC1;
	VSFrame.sync.FrameSync[2] = FRAMESYNC2;
	ScramblePtr = 0;
	RxdBm = -120;
	SquelchOpen = FALSE;
	xTaskCreate( RFControlTask, ( const signed portCHAR * const ) "RF", 
				configMINIMAL_STACK_SIZE, NULL, RFCONTROL_TASK_PRIORITY, ( xTaskHandle * ) NULL );
}


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      ---------------------------                          */
/*                     | R F C o n t r o l T a s k |                         */
/*                      ---------------------------                          */
/*    Task that manages the ADF7021 chip                                     */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void RFControlTask( void *pvParmeters )
{
int i;
U16BIT Rssi;
	Write7021(Reg1.all);	//turns on VCO 
	vTaskDelay( 1 );
	
	Write7021(Reg3.all);	//turn on clocks 
	vTaskDelay( 1 );

	Write7021(Reg2.all);	//init Tx 
	vTaskDelay( 1 );

	//The following is a workaround for the fine cal chip errata problem with xtal freqs>16.7MHz
//1  15  0xC000 000F Override the IF filter gain and bandwidth setting for the IF filter calibration routine.
//2  4  Set the IF filter bandwidth to 18.75 kHz. Keep all other Register 4 settings as recommended in the data sheet.
//3  9  0x0004 0009  Set the IF filter gain to low and AGC to manual mode.
//4  6  //Set the IF_FINE_CAL bit. Set the IF_CAL_DWELL_TIME bits as recommended in the ADF7021 data sheet. Place the lower and upper tones at 84 kHz and 114.7 kHz, respectively, using the formulas for the fine cal filter tones in the data sheet.
//5  5  Set the IF_CAL_COARSE bit. This performs a coarse calibration immediately followed by a fine calibration. Wait until the filter calibration (coarse plus fine) is complete. This typically takes 5.2 ms. Alternatively, users can monitor the FILTER_CAL_COMPLETE signal to ascertain when the filter calibration routine is complete. This signal is programmed to appear on MUXOUT using Register 0.
//6  9  0x0002 31E9  Set the AGC to automatic gain control. This should be done only after the IF filter calibration routine is complete.
//7  15  0x0000 000F  Reset the test mode register.

	Reg15.fld.CalOvride = 3;	//3==override calibration BW and Gain
	Write7021(Reg15.all);

	Reg4.fld.IFBW = 1;			//IF BW = 18.75KHz
	Write7021(Reg4.all); 

	Reg9.fld.AGCMode = 1;		//manual agc
	Write7021(Reg9.all); 
	vTaskDelay( 1 );
	
	Reg6.fld.IFLowDiv = 117;
	Reg6.fld.IFUpDiv = 86;
	Reg6.fld.IFDwell = 52;
	Reg6.fld.IFFineCal = 1;
	Write7021(Reg6.all); 
	vTaskDelay( 1 );

	Reg5.fld.IFDiv = 393;	//Xtal/50000
	Reg5.fld.IFCourseCal = 1;
	Write7021(Reg5.all); 
	vTaskDelay( 100 );
	//End of calibration sequence

	Write7021(Reg11.all); 
	vTaskDelay( 1 );

	Write7021(Reg12.all); 
	vTaskDelay( 1 );

	Write7021(Reg0.all); 
	vTaskDelay( 1 );

	Reg4.fld.IFBW = 0;			//IF BW = 12.5KHz
	Write7021(Reg4.all); 
	vTaskDelay( 1 );


	Reg9.fld.AGCHi = 70;
	Reg9.fld.AGCLow = 30;
	Reg9.fld.LNAGain = 2;
	Reg9.fld.AGCMode = 0;		//auto agc
	Write7021(Reg9.all); 
	vTaskDelay( 1 );

	Reg15.fld.CalOvride = 0;	//0==back to normal test mode
	Reg15.fld.AnaTstMode = 2;	//==RSSI to test pt A
	Write7021(Reg15.all);
	vTaskDelay( 1 );
	
	Reg10.fld.AfcEn = 0;		//enable AFC
	Write7021(Reg10.all); 
	vTaskDelay( 1 );

	Write7021(Reg7.all); 

	TxState = TXSTATE_OFF;
	
	CurrentRxFreq = Read32FRAM(EEADR_RXFREQ);
	if( (CurrentRxFreq<144000000) || (CurrentRxFreq>148000000) )
	{
		CurrentRxFreq = 146555000;
		Write32FRAM(EEADR_RXFREQ, CurrentRxFreq);
	}
	CurrentTxFreq = Read32FRAM(EEADR_TXFREQ);
	if( (CurrentTxFreq<144000000) || (CurrentTxFreq>148000000) )
	{
		CurrentTxFreq = 146555000;
		Write32FRAM(EEADR_TXFREQ, CurrentTxFreq);
	}
	CurrentVolume = (S32BIT)ReadFRAM( EEADR_RXVOL );
	

	SetPTT(FALSE);
	for( ;; )	//loop forever in this task
    {
	 	vTaskDelay( 250 );
		Reg7.fld.ReadSel = 5;	//readback RSSI
	 	Rssi = Read7021();
	 	i = (Rssi>>7)& 0x000F;
	 	RxdBm = -130 + ( DBM_TBL[i] + (Rssi&0x7F) )/2;
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -------------------                                  */
/*                     | W r i t e 7 0 2 1 |                                 */
/*                      -------------------                                  */
/*    Writes a 32 bit value to the ADF7021 chip using the SPI port           */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void Write7021(U32BIT val)
{
	while( !(*AT91C_SPI_SR & AT91C_SPI_TDRE) ){};	// wait for transfer completed
	*AT91C_SPI_TDR = (val>>16)|(0x0<<16); // (SPI) Transmit Data Register
	while( !(*AT91C_SPI_SR & AT91C_SPI_TDRE) ){};	// wait for transfer completed
	*AT91C_SPI_TDR = AT91C_SPI_LASTXFER|(val&0xFFFF)|(0x0<<16); // (SPI) Transmit Data Register
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -----------------                                    */
/*                     | R e a d 7 0 2 1 |                                   */
/*                      -----------------                                    */
/*    Reads back a 16 bit value from the ADF7021 chip using the SPI port     */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static U16BIT Read7021(void)
{
t16BITS tmp;
	//first send the readback cmd(Reg7)
	while( !(*AT91C_SPI_SR & AT91C_SPI_TDRE) ){};	// wait for ok to write to transmitter
	tmp.all = *AT91C_SPI_RDR;		//make sure receive buf is empty
	*AT91C_SPI_TDR = (Reg7.all>>16)|(0x0<<16); // (SPI) Transmit Data Register
	while( !(*AT91C_SPI_SR & AT91C_SPI_RDRF) ){};	// wait for transfer completed
	tmp.all = *AT91C_SPI_RDR;		//make sure receive buf is empty
	*AT91C_SPI_TDR = AT91C_SPI_LASTXFER|(Reg7.all&0xFFFF)|(0x0<<16); // (SPI) Transmit Data Register
	while( !(*AT91C_SPI_SR & AT91C_SPI_RDRF) ){};	// wait for transfer completed
	tmp.all = *AT91C_SPI_RDR;		//make sure receive buf is empty
	*AT91C_SPI_TDR = AT91C_SPI_LASTXFER|(0x1<<16); // (SPI) Transmit Data Register no chip sel 9 msbits
	while( !(*AT91C_SPI_SR & AT91C_SPI_RDRF) ){};	// wait for transfer completed
	tmp.bytes.b1 = (U8BIT)*AT91C_SPI_RDR;
	*AT91C_SPI_TDR = AT91C_SPI_LASTXFER|(0x3<<16); // (SPI) Transmit Data Register no chip sel 8 lsbits
	while( !(*AT91C_SPI_SR & AT91C_SPI_RDRF) ){};	// wait for transfer completed
	tmp.bytes.b0 = (U8BIT)*AT91C_SPI_RDR;
	return tmp.all;
}


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -------------                                        */
/*                     | S e t P T T |                                       */
/*                      -------------                                        */
/*    Sets the Rx/Tx mode of the ADF7021 chip                                */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void SetPTT(BOOL On)
{
	if( TUNE_DISPLAY == DisplayMode)
	{
		Reg15.fld.TxTstMode = 1;	//CW mode
	}
	else
	{
		Reg15.fld.TxTstMode = 0;	//Normal mode
	}
	Write7021(Reg15.all);
	if(On)
	{
		AT91F_AIC_DisableIt (AT91C_BASE_AIC, AT91C_ID_FIQ);
		Reg0.fld.TR = 0;	//Tx
		Reg0.fld.Uart = 1;
		SetTxFrequency(CurrentTxFreq);
		AT91F_PIO_CfgOutput( AT91C_BASE_PIOA, TXDATA_7021 ) ;
		ScramblePtr = 0;
		TxState = TXSTATE_ON;
	}
	else
	{
		TxState = TXSTATE_OFF;
		AT91F_PIO_CfgInput( AT91C_BASE_PIOA, RXCLK_7021 ) ;
		AT91F_AIC_EnableIt (AT91C_BASE_AIC, AT91C_ID_FIQ);
		Reg0.fld.TR = 1;	//Rx
		Reg0.fld.Uart = 0;
		SetRxFrequency(CurrentRxFreq);
		TxState = TXSTATE_OFF;
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -----------------------------                        */
/*                     | S e t T x F r e q u e n c y |                       */
/*                      -----------------------------                        */
/*    Sets the Tx Frequency of the ADF7021 chip                              */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void SetTxFrequency(U32BIT Freq)
{
int F;
int I;
	CurrentTxFreq = Freq;
 	Freq += VCO_OFFSET;
	I = (2*Freq)/ADF7021_PFD_FREQ;
	F = ( Freq - (I*ADF7021_PFD_FREQ/2) ) / ADF7021_FRACT_STEP_FREQ;
	Reg0.fld.Int = I;
	Reg0.fld.Fract = F;
	Write7021(Reg0.all);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -----------------------------                        */
/*                     | S e t R x F r e q u e n c y |                       */
/*                      -----------------------------                        */
/*    Sets the Rx Frequency of the ADF7021 chip                              */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void SetRxFrequency(U32BIT Freq)
{
int F;
int I;
	CurrentRxFreq = Freq;
	Freq = Freq - 100000 + VCO_OFFSET;
	I = (2*Freq)/ADF7021_PFD_FREQ;
	F = ( Freq - (I*ADF7021_PFD_FREQ/2) ) / ADF7021_FRACT_STEP_FREQ;
	Reg0.fld.Int = I;
	Reg0.fld.Fract = F;
	Write7021(Reg0.all);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                         ---------------                                   */
/*                        | S t a r t T x |                                  */
/*                         ---------------                                   */
/*    Called to get everything ready for transmit mode                       */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void StartTx()
{
int i;
	RFTxQHead = 0;
	RFTxQTail = 0;
	EchoPointer = 0;
	InFrame.fld.CW1 = 0x0000;
	InFrame.fld.CW2 = 0x04;		//Encoder(RF Tx mode) enable

	if( ECHO_DISPLAY == DisplayMode)
	{
		//Start with Fixed Sync frame
		PutDstarVSFrame();
		TxSyncFrameCount = 0;
	}
	else
	{
		//Start with Fixed Header frame
		for(i = 0; i<HEADER_LENGTH; i++)
			RFTxDataQueue[RFTxQHead++] = DS_HEADER[i];
		TxSyncFrameCount = FRAMESYNC_SPACING;
	}
	TxShiftCount = 0;
	SetPTT(TRUE);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                         -----------                                       */
/*                        | E n d T x |                                      */
/*                         -----------                                       */
/*    Called to finish up a TX session                                       */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void EndTx()
{
	if( TXSTATE_ON == TxState )
	{
		
		TxState = TXSTATE_FINISHING;
	}
//	else
//	{
//		StartRx();
//	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                         ---------------                                   */
/*                        | S t a r t R x |                                  */
/*                         ---------------                                   */
/*    Called to get everything ready for receive mode                        */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void StartRx()
{
	VSFrame.sync.FrameSync[0] = FRAMESYNC0;
	VSFrame.sync.FrameSync[1] = FRAMESYNC1;
	VSFrame.sync.FrameSync[2] = FRAMESYNC2;
	EchoPointer = 0;
	EchoLength = 0;
	RFRxQHead = 0;
	RFRxQTail = 0;
	InFrame.fld.CW1 = 0x0000;
	InFrame.fld.CW2 = 0x00;		//Decoder(RF Rx mode) enable
	InFrame.fld.Volume = 0;		//Decoder min volume
	SetPTT(FALSE);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                  -------------------------------                          */
/*                 | P u t D s t a r V D F r a m e |                         */
/*                  -------------------------------                          */
/*    Called to create dstar voice/data frame and put in RF data FIFO        */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void PutDstarVDFrame()
{
int i;
t16BITS tmp;
	if( ECHO_DISPLAY == DisplayMode)
	{
		VDFrame.Buf32[0] = EchoBuffer[EchoPointer++];
		VDFrame.Buf32[1] = EchoBuffer[EchoPointer++];
		VDFrame.Buf32[2] = EchoBuffer[EchoPointer++];
		if(EchoPointer>ECHO_BUF_LENGTH)
		{
			EchoPointer = ECHO_BUF_LENGTH;
			EchoLength = 0;
		}
		EchoLength -= 3;
		if(EchoLength <= 0)
			EchoLength = 0;
	}
	else
	{
		tmp.all = OutFrame.fld.ChData[0];
		VDFrame.data.VData[0] = tmp.bytes.b1;
		VDFrame.data.VData[1] = tmp.bytes.b0;
		tmp.all = OutFrame.fld.ChData[1];
		VDFrame.data.VData[2] = tmp.bytes.b1;
		VDFrame.data.VData[3] = tmp.bytes.b0;
		tmp.all = OutFrame.fld.ChData[2];
		VDFrame.data.VData[4] = tmp.bytes.b1;
		VDFrame.data.VData[5] = tmp.bytes.b0;
		tmp.all = OutFrame.fld.ChData[3];
		VDFrame.data.VData[6] = tmp.bytes.b1;
		VDFrame.data.VData[7] = tmp.bytes.b0;
		tmp.all = OutFrame.fld.ChData[4];
		VDFrame.data.VData[8] = tmp.bytes.b1;

		VDFrame.data.UsrData[0] = 0x66 ^ SCRAMBLE_TBL[0];
		VDFrame.data.UsrData[1] = 0x66 ^ SCRAMBLE_TBL[1];
		VDFrame.data.UsrData[2] = 0x66 ^ SCRAMBLE_TBL[2];
	}
	if(	TXSTATE_FINISHING == TxState )
	{
		VDFrame.end.EndSync[0] = ENDSYNC0;
		VDFrame.end.EndSync[1] = ENDSYNC1;
		VDFrame.end.EndSync[2] = ENDSYNC2;
	}
	
	for(i=0; i<VDATAFRAME_SIZE; i++)
	{	//stick in rf tx queue
		RFTxDataQueue[RFTxQHead++] = VDFrame.Buf32[i];
		RFTxQHead = RFTxQHead & (RFTQ_SIZE-1);
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                  -------------------------------                          */
/*                 | P u t D s t a r V S F r a m e |                         */
/*                  -------------------------------                          */
/*    Called to create dstar voice/sync frame and put in RF data FIFO        */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void PutDstarVSFrame()
{
int i;
t16BITS tmp;
	if( ECHO_DISPLAY == DisplayMode)
	{
		VSFrame.Buf32[0] = EchoBuffer[EchoPointer++];
		VSFrame.Buf32[1] = EchoBuffer[EchoPointer++];
		VSFrame.Buf32[2] = EchoBuffer[EchoPointer++];
		if(EchoPointer>ECHO_BUF_LENGTH)
		{
			EchoPointer = ECHO_BUF_LENGTH;
			EchoLength = 0;
		}
		EchoLength -= 3;
		if(EchoLength <= 0)
			EchoLength = 0;
	}
	else
	{
		tmp.all = OutFrame.fld.ChData[0];
		VSFrame.sync.VData[0] = tmp.bytes.b1;
		VSFrame.sync.VData[1] = tmp.bytes.b0;
		tmp.all = OutFrame.fld.ChData[1];
		VSFrame.sync.VData[2] = tmp.bytes.b1;
		VSFrame.sync.VData[3] = tmp.bytes.b0;

		tmp.all = OutFrame.fld.ChData[2];
		VSFrame.sync.VData[4] = tmp.bytes.b1;
		VSFrame.sync.VData[5] = tmp.bytes.b0;
		tmp.all = OutFrame.fld.ChData[3];
		VSFrame.sync.VData[6] = tmp.bytes.b1;
		VSFrame.sync.VData[7] = tmp.bytes.b0;
		tmp.all = OutFrame.fld.ChData[4];
		VSFrame.sync.VData[8] = tmp.bytes.b1;
	}
	
	ScramblePtr = 0;
	for(i=0; i<VDATAFRAME_SIZE; i++)
	{	//stick in rf tx queue
		RFTxDataQueue[RFTxQHead++] = VSFrame.Buf32[i];
		RFTxQHead = RFTxQHead & (RFTQ_SIZE-1);
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                  -------------------------------                          */
/*                 | G e t D s t a r V D F r a m e |                         */
/*                  -------------------------------                          */
/*    Called to get dstar voice/data frame from RF Rx data FIFO              */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void GetDstarVDFrame()
{
int i;
t16BITS tmp;
	//copy to VDF frame bufffer
	VDFrame.Buf32[0] = RFRxDataQueue[RFRxQTail].Buf32[0];
	VDFrame.Buf32[1] = RFRxDataQueue[RFRxQTail].Buf32[1];
	VDFrame.Buf32[2] = RFRxDataQueue[RFRxQTail++].Buf32[2];
	RFRxQTail = RFRxQTail & (RFRQ_SIZE-1);
	if( SquelchOpen && ECHO_DISPLAY == DisplayMode)
	{
		EchoBuffer[EchoPointer++] = VDFrame.Buf32[0];
		EchoBuffer[EchoPointer++] = VDFrame.Buf32[1];
		EchoBuffer[EchoPointer++] = VDFrame.Buf32[2];
		if(EchoPointer>ECHO_BUF_LENGTH)
			EchoPointer = 0;
	}
	tmp.bytes.b1 = VDFrame.data.VData[0];
	tmp.bytes.b0 = VDFrame.data.VData[1];
	InFrame.fld.ChData[0] = tmp.all;
	tmp.bytes.b1 = VDFrame.data.VData[2];
	tmp.bytes.b0 = VDFrame.data.VData[3];
	InFrame.fld.ChData[1] = tmp.all;
	tmp.bytes.b1 = VDFrame.data.VData[4];
	tmp.bytes.b0 = VDFrame.data.VData[5];
	InFrame.fld.ChData[2] = tmp.all;
	tmp.bytes.b1 = VDFrame.data.VData[6];
	tmp.bytes.b0 = VDFrame.data.VData[7];
	InFrame.fld.ChData[3] = tmp.all;
	tmp.bytes.b1 = VDFrame.data.VData[8];
	tmp.bytes.b0 = 0;
	InFrame.fld.ChData[4] = tmp.all;
	if( (FRAMESYNC2 == VDFrame.sync.FrameSync[2] ) && (FRAMESYNC1 == VDFrame.sync.FrameSync[1] ) )
	{
		SendUsart( '\n' );SendUsart( '\r' );SendUsart( 'S' );
		SendUsart( 'y' );SendUsart( 'n' );SendUsart( 'c' );
	}
	else
	{
		if(SquelchOpen)
		{
			SendUsart( '\n' );SendUsart( '\r' );
			SendUsart( VDFrame.data.UsrData[0]^SCRAMBLE_TBL[0] );
			SendUsart( VDFrame.data.UsrData[1]^SCRAMBLE_TBL[1] );
			SendUsart( VDFrame.data.UsrData[2]^SCRAMBLE_TBL[2] );
		}
	}
}

