/* ==========================================================================*/
/* - - - - - - - - - - - - - -  U s a r t . C   - - - - - - - - - - - - - - -*/
/* ==========================================================================*/
/*	Created   2007-03-10   M. Wheatley                                       */
/*	Modified  2007-03-10   MW                                                */
/*.........................................................................  */
/* VHF Digital Voice Xceiver AT91SAM7256 usart handler                       */

/*---------------------------------------------------------------------------*/
/*------------------------> I N C L U D E S <--------------------------------*/
/*---------------------------------------------------------------------------*/
#include "Board.h"
#include "usart.h"
#include "FPcom.h"
#include "irq.h"
#include <stdio.h>
#include <string.h>
 
/*---------------------------------------------------------------------------*/
/*--------------------> L O C A L   D E F I N E S <--------------------------*/
/*---------------------------------------------------------------------------*/
#define USART_TASK_PRIORITY ( tskIDLE_PRIORITY + 2 )
#define QUEUE_SIZE 3

#define PARSE_TOP 0
#define PARSE_FREQ 1
//* Standard Asynchronous Mode : 8 bits , 1 stop , no parity
#define ASYNC_MODE ( AT91C_US_USMODE_NORMAL + \
                        AT91C_US_NBSTOP_1_BIT + \
                        AT91C_US_PAR_NONE + \
                        AT91C_US_CHRL_8_BITS + \
                        AT91C_US_CLKS_CLOCK )

/*---------------------------------------------------------------------------*/
/*-------------------> M E M O R Y  A L L O C A T I O N <--------------------*/
/*---------------------------------------------------------------------------*/
U8BIT UsartRxQue[RXQ_SIZE];
U8BIT UsartTxQue[TXQ_SIZE];
volatile U32BIT RxHead;
U32BIT RxTail;
U32BIT TxHead;
volatile U32BIT TxTail;
xSemaphoreHandle xUartSemaphore;
S32BIT DebugValue1;
S32BIT DebugValue2;

static U32BIT ParseLevel;
static U32BIT ParseIndex;
char ChBuf[80];

/*---------------------------------------------------------------------------*/
/*--------------------------> P R O T O T Y P E S <--------------------------*/
/*---------------------------------------------------------------------------*/
void UsartTask( void *pvParmeters );
void SendUsartStr(U8BIT* Buf);
void SendUsartStrWait(U8BIT* Buf);
void SendUsart(U8BIT Ch);
void ParseRx(U8BIT Ch);
static void ItoA( char* str, int val);
static void XtoA( char* str, int val);

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                  -----------------------------                            */
/*                 | D e b u g U s a r t I n i t |                           */
/*                  -----------------------------                            */
/*    Called to initialize the USART 0 port.                                 */
/*			parity 0,1,2 == n,o,e                                            */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void UsartInit( U32BIT Rate, U32BIT Parity )
{
	ParseLevel = PARSE_TOP;
	if( 0 == Rate)
	{
		*AT91C_US0_CR = AT91C_US_TXDIS | AT91C_US_RXDIS;
		*AT91C_US0_IDR = AT91C_US_RXRDY | AT91C_US_TXRDY;
		RxHead = 0;		//init Queue pointers
		RxTail = 0;
		TxHead = 0;
		TxTail = 0;
	}
	else
	{
		vSemaphoreCreateBinary( xUartSemaphore );
		xTaskCreate( UsartTask, ( const signed portCHAR * const ) "USART", 
					configMINIMAL_STACK_SIZE, NULL, USART_TASK_PRIORITY, ( xTaskHandle * ) NULL );
 		//* Configure PIO controllers to periph mode
 		AT91F_PIO_CfgPeriph( AT91C_BASE_PIOA,
 			((unsigned int) RXD    ) |
 			((unsigned int) TXD    ),// Peripheral A
 			0); // Peripheral B
   		AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1 << AT91C_ID_US0 ) ;
		// Usart Configure
 		AT91C_BASE_US0->US_BRGR = MCK/(16*Rate) + 1;
		AT91C_BASE_US0->US_MR = ASYNC_MODE;
		AT91C_BASE_US0->US_TTGR = 0;
		// Enable usart
		AT91C_BASE_US0->US_CR = AT91C_US_RXEN | AT91C_US_TXEN;

    	//* Enable USART IT error and RXRDY
		AT91F_US_EnableIt(AT91C_BASE_US0,AT91C_US_TIMEOUT | AT91C_US_FRAME | AT91C_US_OVRE |AT91C_US_RXRDY);
		RxHead = 0;		//init Queue pointers
		RxTail = 0;
		TxHead = 0;
		TxTail = 0;
	    //* Config Usart interrupt
		AT91F_AIC_ConfigureIt ( AT91C_BASE_AIC, AT91C_ID_US0, USART_INTERRUPT_LEVEL,
								AT91C_AIC_SRCTYPE_INT_LEVEL_SENSITIVE, UsartIrqService);
		AT91F_AIC_EnableIt (AT91C_BASE_AIC, AT91C_ID_US0);
	}
}


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -------------------                                  */
/*                     | U s a r t T a s k |                                 */
/*                      -------------------                                  */
/*    Task that process usart rx bytes                                       */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void UsartTask( void *pvParmeters )
{
    for( ;; )	//loop forever in this task
    {
        if( xSemaphoreTake( xUartSemaphore, ( portTickType )500 ) == pdTRUE )
		{
			while( RxHead != RxTail )
			{
				ParseRx( UsartRxQue[RxTail++] );
				RxTail &= (RXQ_SIZE-1);	//deal with rollover
			}
		}
		else
		{
//			SendUsartStr((U8BIT*)"\n\rTest");
		}
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -------------------------                            */
/*                     | S e n d U s a r t S t r |                           */
/*                      -------------------------                            */
/*    Function that sends a buffer string to the usart                       */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void SendUsartStr(U8BIT* Buf)
{
U32BIT i = 0;
	*AT91C_US0_IDR = AT91C_US_TXRDY;
	while( Buf[i] )
	{
		UsartTxQue[TxHead++] = Buf[i++];
		TxHead &= (TXQ_SIZE-1);
	}
	*AT91C_US0_IER = AT91C_US_TXRDY;
 }

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                   ---------------------------------                       */
/*                  | S e n d U s a r t S t r W a i t |                      */
/*                   ---------------------------------                       */
/*    Function that sends a buffer string to the usart and waits to finish   */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void SendUsartStrWait(U8BIT* Buf)
{
U32BIT i = 0;
	*AT91C_US0_IDR = AT91C_US_TXRDY;
	while( Buf[i] )
	{
		*AT91C_US0_THR = Buf[i++];
		while( !(*AT91C_US0_CSR & AT91C_US_TXEMPTY) ){};
	}
	*AT91C_US0_IER = AT91C_US_TXRDY;
}


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -------------------                                  */
/*                     | S e n d U s a r t |                                 */
/*                      -------------------                                  */
/*    Function that sends a buffer to the usart                              */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void SendUsart(U8BIT Ch)
{
	*AT91C_US0_IDR = AT91C_US_TXRDY;
	UsartTxQue[TxHead++] = Ch;
	TxHead &= (TXQ_SIZE-1);
	if(TxHead!=TxTail)
		*AT91C_US0_IER = AT91C_US_TXRDY;
 }


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      ---------------                                      */
/*                     | P a r s e R x |                                     */
/*                      ---------------                                      */
/*    Parse incoming characters                                              */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void ParseRx(U8BIT Ch)
{
	SendUsart(Ch);
	switch(	ParseLevel )	//enter into parsing stages
	{
		case PARSE_TOP:
			switch(Ch)
			{
				default:
					break;
			}
			break;
		default:
			break;
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                  ---------------------                                    */
/*                 | P r i n t D e b u g |                                   */
/*                  ---------------------                                    */
/*  Print debug  prefix string 'str', integer value, Hex==1  decimal==0      */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void PrintUsartDebug( char* str, int val, int HexMode)
{
	strcpy(ChBuf, str);
	if(HexMode)
		XtoA( ChBuf, val);
	else
		ItoA( ChBuf, val);
	SendUsartStr((U8BIT*)ChBuf);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                           ---------                                       */
/*                          | I t o A |                                      */
/*                           ---------                                       */
/* Append an integer 'val' in decimal to the string 'str'                    */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void ItoA( char* str, int val)
{
BOOL Leadzero;
int i,j;
unsigned int x;
unsigned char ch;
	i = 0;
	while( 0 != (ch=str[i]) )	//go to string end
	{
		i++;
	}
	if(val<0)
	{
		val = -val;
		str[i++] = '-';
	}
	Leadzero = TRUE;
	ch = val/1000000000; val = val - 1000000000*ch;
	if(ch) Leadzero = FALSE;
	if(!Leadzero) {	ch += '0'; str[i++] = ch; }
	ch = val/100000000;	val = val - 100000000*ch;
	if(ch) Leadzero = FALSE;
	if(!Leadzero) {	ch += '0'; str[i++] = ch; }
	ch = val/10000000; val = val - 10000000*ch;
	if(ch) Leadzero = FALSE;
	if(!Leadzero) {	ch += '0'; str[i++] = ch; }
	ch = val/1000000; val = val - 1000000*ch;
	if(ch) Leadzero = FALSE;
	if(!Leadzero) {	ch += '0'; str[i++] = ch; }
	ch = val/100000; val = val - 100000*ch;
	if(ch) Leadzero = FALSE;
	if(!Leadzero) {	ch += '0'; str[i++] = ch; }
	ch = val/10000; val = val - 10000*ch;
	if(ch) Leadzero = FALSE;
	if(!Leadzero) {	ch += '0'; str[i++] = ch; }
	ch = val/1000; val = val - 1000*ch;
	if(ch) Leadzero = FALSE;
	if(!Leadzero) {	ch += '0'; str[i++] = ch; }
	ch = val/100; val = val - 100*ch;
	if(ch) Leadzero = FALSE;
	if(!Leadzero) {	ch += '0'; str[i++] = ch; }
	ch = val/10; val = val - 10*ch;
	if(ch) Leadzero = FALSE;
	if(!Leadzero) {	ch += '0'; str[i++] = ch; }
	ch = val;
	ch += '0';
	str[i++] = ch;
	str[i] = 0;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                           ---------                                       */
/*                          | X t o A |                                      */
/*                           ---------                                       */
/* Append an integer 'val' in HEX to the string 'str'                        */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void XtoA( char* str, int val)
{
int i,j;
unsigned int x;
unsigned char ch;
	i = 0;
	while( 0 != (ch=str[i]) )	//go to string end
		i++;
	str[i++] = 'x';
	for(j=0; j<8; j++)
	{
		x = (unsigned int)(val&0xF0000000);
		x >>= 28;
		ch = (char)x;
		if(ch<10)
			ch += '0';
		else
			ch += ('A'-10);
		str[i++] = ch;
		if(i==3)
			str[i++] =' ';
		val = val<<4;
	}
	str[i] = 0;
}
