/* ==========================================================================*/
/* - - - - - - - - - - - - - -  M a i n . C   - - - - - - - - - - - - - - - -*/
/* ==========================================================================*/
/*	Created   2007-03-10   M. Wheatley                                       */
/*	Modified  2007-03-10   MW                                                */
/*.........................................................................  */
/* VHF Digital Voice Xceiver AT91SAM7256 Firmware                            */

/*---------------------------------------------------------------------------*/
/*------------------------> I N C L U D E S <--------------------------------*/
/*---------------------------------------------------------------------------*/
#include "Board.h"
#include "common.h"
#include "usart.h"
#include "timer.h"
#include "Fpcom.h"
#include "Codec.h"
#include "rfcontrol.h"
#include "ambe2020.h"
#include "Irq.h"

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"


/*---------------------------------------------------------------------------*/
/*--------------------> L O C A L   D E F I N E S <--------------------------*/
/*---------------------------------------------------------------------------*/
#define LED_TASK_PRIORITY ( tskIDLE_PRIORITY + 3 )

#define QUEUE_SIZE 3

/*---------------------------------------------------------------------------*/
/*-------------------> M E M O R Y  A L L O C A T I O N <--------------------*/
/*---------------------------------------------------------------------------*/
static U8BIT Val;

/*---------------------------------------------------------------------------*/
/*------------------> L O C A L    P R O T O T Y P E S <---------------------*/
/*---------------------------------------------------------------------------*/
void vApplicationIdleHook( void );
void LedTask( void *pvParmeters );

/*---------------------------------------------------------------------------*/
/*-------------------> M E M O R Y  A L L O C A T I O N <--------------------*/
/*---------------------------------------------------------------------------*/

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                             ---------                                     */
/*                            | M a i n |                                    */
/*                             ---------                                     */
/*    This is the program entry point.                                       */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
int main( void )
{
	Val = 0;
	AT91C_BASE_AIC->AIC_EOICR = 0;
    // enable the clock of the PIO
	AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1 << AT91C_ID_PIOA ) ;
	AT91F_PIO_CfgPullup(AT91C_BASE_PIOA, ~(UPSW_|DNSW_|MODESW_|MISO7021|RXCLK_7021|RXDATA_7021));	//no int pullups!
	AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LED1|LED2 ) ;
	AT91F_PIO_SetOutput( AT91C_BASE_PIOA, TP5 ) ;
	AT91F_PIO_CfgOutput( AT91C_BASE_PIOA, LED1|LED2|TP5 ) ;

	TimerInit();
	UsartInit(19200,0);
	CodecInit();
	Ambe2020Init();
	FPComInit();
	RFControlInit();
	xTaskCreate( LedTask, ( const signed portCHAR * const ) "LED", 
					configMINIMAL_STACK_SIZE, NULL, LED_TASK_PRIORITY, ( xTaskHandle * ) NULL );
	vTaskStartScheduler();
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                             ---------------                               */
/*                            | L e d T a s k |                              */
/*                             ---------------                               */
/*    This tasks blinks the LED and resets the overload latch.               */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void LedTask( void *pvParmeters )
{
U8BIT vol=0;
    for( ;; )
    {
		if( TXSTATE_OFF != TxState )
		{
			AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LED1 ) ;
			AT91F_PIO_SetOutput( AT91C_BASE_PIOA, LED2 );
			vTaskDelay(9);
			AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LED2 );
			vTaskDelay(1);
		}
		else
		{
			if(SquelchOpen)
			{
				AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LED2 ) ;
				AT91F_PIO_SetOutput( AT91C_BASE_PIOA, LED1 );
				vTaskDelay(50);
				AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LED1 );
				AT91F_PIO_SetOutput( AT91C_BASE_PIOA, LED2 ) ;
				vTaskDelay(50);
			}
			else
			{
				AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LED2 ) ;
				AT91F_PIO_SetOutput( AT91C_BASE_PIOA, LED1 );
				vTaskDelay(9);
				AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LED1 );
				vTaskDelay(1);
			}
		}
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                 -----------------------------------------                 */
/*                | v A p p l i c a t i o n I d l e H o o k |                */
/*                 -----------------------------------------                 */
/*    This is called during RTOS idle time.                                  */
/*    Maybe good place for watchdog petting.                                 */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void vApplicationIdleHook( void )
{
}
