/* ==========================================================================*/
/* - - - - - - - - - - - - -  T i m e r . c  - - - - - - - - - - - - - - - - */
/* ==========================================================================*/
/*	Created   2007-03-10   M. Wheatley                                       */
/*	Modified  2007-03-10   MW                                                */
/*.........................................................................  */
/* VHF Digital Voice Xceiver AT91SAM7256 Timer management                    */

/*---------------------------------------------------------------------------*/
/*------------------------> I N C L U D E S <--------------------------------*/
/*---------------------------------------------------------------------------*/
#include "Board.h"
#include "common.h"
#include "timer.h"
#include "Irq.h"

/*---------------------------------------------------------------------------*/
/*--------------------> L O C A L   D E F I N E S <--------------------------*/
/*---------------------------------------------------------------------------*/


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      -------------------                                  */
/*                     | T i m e r I n i t |                                 */
/*                      -------------------                                  */
/*    Called to initialize the Timers.                                       */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void TimerInit ( void )
{
int dummy;
    //* Open timer0  (Provides 12.288MHz clock for Codec)
	//* First, enable the clock of the TIMER
 	AT91F_PIO_CfgPeriph( AT91C_BASE_PIOA,
 		0 , // Peripheral A
 		CLK_12288); // Peripheral B
	AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1<<AT91C_ID_TC0 ) ;
	//* Disable the clock and the interrupts
	AT91C_BASE_TC0->TC_CCR = AT91C_TC_CLKDIS;
	AT91C_BASE_TC0->TC_IDR = 0xFFFFFFFF;
	//* Clear status bit
	dummy = AT91C_BASE_TC0->TC_SR;
	//* Set the Mode of the Timer Counter
	AT91C_BASE_TC0->TC_CMR = AT91C_TC_CLKS_TIMER_DIV1_CLOCK|AT91C_TC_WAVESEL_UP_AUTO|AT91C_TC_WAVE|AT91C_TC_ACPA_TOGGLE; // MCLK/2
	//* Enable the clock
	AT91C_BASE_TC0->TC_CCR = AT91C_TC_CLKEN;
	AT91C_BASE_TC0->TC_RA = 1;
	AT91C_BASE_TC0->TC_RC = 1;
	AT91C_BASE_TC0->TC_CCR = AT91C_TC_SWTRG;


    //* Open timer1	(Message Timer)
	//* First, enable the clock of the TIMER
	AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1<<AT91C_ID_TC1 ) ;
	//* Disable the clock and the interrupts
	AT91C_BASE_TC1->TC_CCR = AT91C_TC_CLKDIS;
	AT91C_BASE_TC1->TC_IDR = 0xFFFFFFFF;
	//* Clear status bit
	dummy = AT91C_BASE_TC1->TC_SR;
	//* Set the Mode of the Timer Counter
	AT91C_BASE_TC1->TC_CMR = AT91C_TC_CLKS_TIMER_DIV3_CLOCK; //~23Hz
	//* Enable the clock
	AT91C_BASE_TC1->TC_CCR = AT91C_TC_CLKEN;
    //* Config Timer 1 interrupt
	AT91F_AIC_ConfigureIt ( AT91C_BASE_AIC, AT91C_ID_TC1, TIMER1_INTERRUPT_LEVEL,
						AT91C_AIC_SRCTYPE_INT_LEVEL_SENSITIVE, Timer1IrqService);
	AT91C_BASE_TC1->TC_IER = AT91C_TC_CPCS;  //  IRQ enable CPC
	AT91F_AIC_EnableIt (AT91C_BASE_AIC, AT91C_ID_TC1);
	//* Start timer1
	AT91C_BASE_TC1->TC_CCR = AT91C_TC_SWTRG;
	
   //* Open timer2	(AMBE Message One Shot Timer)
	//* First, enable the clock of the TIMER
	AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1<<AT91C_ID_TC2 );
	//* Disable the clock and the interrupts
	AT91C_BASE_TC2->TC_CCR = AT91C_TC_CLKDIS;
	AT91C_BASE_TC2->TC_IDR = 0xFFFFFFFF;
	//* Clear status bit
	dummy = AT91C_BASE_TC2->TC_SR;
	//* Set the Mode of the Timer Counter
	AT91C_BASE_TC2->TC_RC = 0xFFFF;
	AT91C_BASE_TC2->TC_CMR = AT91C_TC_CLKS_TIMER_DIV5_CLOCK|AT91C_TC_CPCSTOP|AT91C_TC_WAVESEL_UP_AUTO|
								AT91C_TC_WAVE; //48000 hz clk to timer2
	//* Enable the clock
	AT91C_BASE_TC2->TC_CCR = AT91C_TC_CLKEN;
    //* Config Timer 2 interrupt
	AT91F_AIC_ConfigureIt ( AT91C_BASE_AIC, AT91C_ID_TC2, TIMER2_INTERRUPT_LEVEL,
						AT91C_AIC_SRCTYPE_INT_LEVEL_SENSITIVE, Timer2IrqService);
	AT91C_BASE_TC2->TC_IER = AT91C_TC_CPCS;  //  IRQ enable CPC
	AT91F_AIC_EnableIt (AT91C_BASE_AIC, AT91C_ID_TC2);
	
	//* Setup PWM channel 2 (Used to generate a 4800Hz data clock interrupt)
	//* enable the clock of the PWM system
	AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1<<AT91C_ID_PWMC ) ;

	*AT91C_PWMC_DIS = AT91C_PWMC_CHID2;
	*AT91C_PWMC_MR = 0;	//turn off DIVA and B
	*AT91C_PWMC_CH2_CMR  = AT91C_PWMC_CPRE_MCK_1024|AT91C_PWMC_CPD; 	// (PWMC_CH2) MCLK/1024 update period
	*AT91C_PWMC_CH2_CPRDR = 10; 	// (PWMC_CH2) Channel Period Register 4800Hz
	*AT91C_PWMC_CH2_CDTYR = 1; 		// (PWMC_CH2) Channel Duty Cycle Register
	*AT91C_PWMC_CH2_CUPDR  = 10; 	// (PWMC_CH2) Channel Update Register
    //* Config PWM period 4800Hz interrupt
	AT91F_AIC_ConfigureIt ( AT91C_BASE_AIC, AT91C_ID_PWMC, PWM_INTERRUPT_LEVEL,
						AT91C_AIC_SRCTYPE_INT_LEVEL_SENSITIVE, PWM2IrqService);
	AT91C_BASE_PWMC->PWMC_IER = AT91C_PWMC_CHID2;  //  IRQ enable CPC
	AT91F_AIC_EnableIt (AT91C_BASE_AIC, AT91C_ID_PWMC);

	*AT91C_PWMC_ENA = AT91C_PWMC_CHID2;
	
}
