/* ==========================================================================*/
/* - - - - - - - - - - - -   A D F 7 0 2 1 . H   - - - - - - - - - - - - - - */
/* ==========================================================================*/
/*    Definition File for the Analog Devices ADF7021 RF Xceiver Chip         */
/*  Created   2007-03-15                                                     */
/*  Modified  2007-03-15                                                     */
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
#ifndef ADF7021_H_
#define ADF7021_H_

/*---------------------------------------------------------------------------*/
/*-------------------------> D E F I N E S <---------------------------------*/
/*---------------------------------------------------------------------------*/
#define ADF7021_XTAL_FREQ (19660800)
#define ADF7021_PFD_FREQ (ADF7021_XTAL_FREQ/3)
#define ADF7021_FRACT_STEP_FREQ ( ADF7021_PFD_FREQ/(2*32768) )

/*---------------------------------------------------------------------------*/
/*--------> T Y P E D E F S   U N I O N S    S T R U C T U R E S <-----------*/
/*---------------------------------------------------------------------------*/
//highly non portable way to access bytes of int
typedef union
{
	struct
	{
		int Reg:4;
		int Fract:15;
		int Int:8;
		int TR:1;
		int Uart:1;
		int MuxOut:3;
	}fld;
	U32BIT all;
}tADF721_R0;

typedef union
{
	struct
	{
		int Reg:4;
		int Rcnt:3;
		int CoutDiv:4;
		int X2:1;
		int OscEn:1;
		int Xbias:2;
		int Cpi:2;
		int VcoEn:1;
		int VcoDiv2:1;
		int VcoBias:4;
		int VcoAdj:2;
		int VcoLext:1;
		int Fill:6;
	}fld;
	U32BIT all;
}tADF721_R1;

typedef union
{
	struct
	{
		int Reg:4;
		int ModSch:3;
		int PaEn:1;
		int PwrRamp:3;
		int PaBias:2;
		int Power:6;
		int TxDev:9;
		int TxInv:2;
		int Narrow:1;
		int Fill:1;
	}fld;
	U32BIT all;
}tADF721_R2;

typedef union
{
	struct
	{
		int Reg:4;
		int BClkDiv:2;
		int DemClkDiv:4;
		int CDRDiv:8;
		int SeqDiv:8;
		int AgcDiv:6;
	}fld;
	U32BIT all;
}tADF721_R3;

typedef union
{
	struct
	{
		int Reg:4;
		int DemSch:3;
		int Dot:1;
		int RxInv:2;
		int DiscBW:10;
		int PostBW:10;
		int IFBW:2;
	}fld;
	U32BIT all;
}tADF721_R4;

typedef union
{
	struct
	{
		int Reg:4;
		int IFCourseCal:1;
		int IFDiv:9;
		int IFAdj:6;
		int IRPhzMag:4;
		int QPhzAdj:1;
		int IRGainMag:5;
		int QGainAdj:1;
		int IRAdjAtn:1;
	}fld;
	U32BIT all;
}tADF721_R5;

typedef union
{
	struct
	{
		int Reg:4;
		int IFFineCal:1;
		int IFLowDiv:8;
		int IFUpDiv:8;
		int IFDwell:7;
		int Fill:4;
	}fld;
	U32BIT all;
}tADF721_R6;

typedef union
{
	struct
	{
		int Reg:4;
		int ADCMode:2;
		int ReadSel:3;
		int Fill:23;
	}fld;
	U32BIT all;
}tADF721_R7;

typedef union
{
	struct
	{
		int Reg:4;
		int SynEn:1;
		int rsrv:1;
		int LnaEn:1;
		int FilEn:1;
		int ADCEn:1;
		int DemEn:1;
		int LogEn:1;
		int TRswEn:1;
		int PAEn:1;
		int DemRst:1;
		int CdrRst:1;
		int CntRst:1;
		int Fill:16;
	}fld;
	U32BIT all;
}tADF721_R8;

typedef union
{
	struct
	{
		int Reg:4;
		int AGCLow:7;
		int AGCHi:7;
		int AGCMode:2;
		int LNAGain:2;
		int FiltGain:2;
		int FiltI:1;
		int LnaMode:1;
		int LnaBias:2;
		int MixLin:1;
		int Fill:3;
	}fld;
	U32BIT all;
}tADF721_R9;

typedef union
{
	struct
	{
		int Reg:4;
		int AfcEn:1;
		int AfcScale:12;
		int KI:4;
		int KP:3;
		int AfcRange:8;
	}fld;
	U32BIT all;
}tADF721_R10;

typedef union
{
	struct
	{
		int Reg:4;
		int SyncLen:2;
		int SyncTol:2;
		int SyncWord:24;
	}fld;
	U32BIT all;
}tADF721_R11;

typedef union
{
	struct
	{
		int Reg:4;
		int LockThresh:2;
		int LockPin:2;
		int DataLen:8;
		int Fill:16;
	}fld;
	U32BIT all;
}tADF721_R12;

typedef union
{
	struct
	{
		int Reg:4;
		int SliceThres:7;
		int VitEn:1;
		int HangRecover:1;
		int VitPathMem:2;
		int CDRThres:7;
		int PreambTime:4;
		int Fill:6;
	}fld;
	U32BIT all;
}tADF721_R13;

typedef union
{
	struct
	{
		int Reg:4;
		int TstEn:1;
		int DacOffset:16;
		int DacGn:4;
		int PulseExt:2;	//?? data sheet
		int LeakFact:3;
		int PkResp:2;	//?? data sheet
	}fld;
	U32BIT all;
}tADF721_R14;

typedef union
{
	struct
	{
		int Reg:4;
		int RxTstMode:4;
		int TxTstMode:3;
		int SigDTstMode:3;
		int PfdTstMode:3;
		int ClkMux:3;
		int PllTstMode:4;
		int AnaTstMode:4;
		int LdHi:1;
		int PwrDown:1;
		int CalOvride:2;
	}fld;
	U32BIT all;
}tADF721_R15;

#endif /*ADF7021_H_*/
