/* ==========================================================================*/
/* - - - - - - - - - - - - -   B o a r d . h   - - - - - - - - - - - - - - - */
/* ==========================================================================*/
/*    Definition File for the VHFDVX  board                                  */
/*  Created   2007-03-09   msw                                               */
/*	Modified  2007-03-09   MW                                                */
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
#ifndef Board_h
#define Board_h
#include "Common.h"

#include "AT91SAM7S256.h"
#define __inline extern inline
#include "lib_AT91SAM7S256.h"
#define __ramfunc

#define AT91C_US_ASYNC_MODE ( AT91C_US_USMODE_NORMAL + AT91C_US_NBSTOP_1_BIT + AT91C_US_PAR_NONE + AT91C_US_CHRL_8_BITS + AT91C_US_CLKS_CLOCK )


/*---------------------------*/
/*  Memories Definition      */
/*---------------------------*/
// The AT91SAM7S256 has a 64-Kbyte SRAM bank, and 256K-Byte Flash

#define  INT_SARM           0x00200000
#define  INT_SARM_REMAP	    0x00000000

#define  INT_FLASH          0x00000000
#define  INT_FLASH_REMAP    0x00100000

#define  BOOTFLASH_START    0x00000000
#define  BOOTFLASH_END    	0x00003FFF

#define  FLASH_PAGE_NB	    1024
#define  FLASH_PAGE_LOCK    64
#define  FLASH_PAGE_SIZE    256

/*---------------------*/
/* I/O Pin Definitions */
/*---------------------*/
/*----------------------------------------------------------------------------------*/
/*  PORT A I/O Pin Definitions                    AT91SAM7 PIO Multiplexing         */
/*                                      PIO     Flash    Periph A   Periph B   PIN  */
/*----------------------------------------------------------------------------------*/
#define CLK_12288       (1<<0)  /*       PA0 / PGMEN0     PWM0      TIOA0     48    */
#define LED1            (1<<1)  /*       PA1 / PGMEN1     PWM1      TIOB0     47    */
#define LED2            (1<<2)  /*       PA2              PWM2      SCK0      44    */
#define SDA             (1<<3)  /*       PA3              TWD       NPCS3     43    */
#define SCL             (1<<4)  /*       PA4              TWCK      TCLK0     36    */
#define RXD             (1<<5)  /*       PA5              RXD0      NPCS3     35    */
#define TXD             (1<<6)  /*       PA6              TXD0      PCK0      34    */
#define EPR_            (1<<7)  /*       PA7              RTS0      PWM3      32    */
#define TP5             (1<<8)  /*       PA8              CTS0      ADTRG     31    */
#define MODESW_         (1<<9)	/*       PA9              DRXD      NPCS1     30    */
#define PTT_            (1<<10) /*       PA10             DTXD      NPCS2     29    */
#define CS7021_         (1<<11) /*       PA11             NPCS0     PWM0      28    */
#define MISO7021        (1<<12) /*       PA12             MISO      PWM1      27    */
#define MOSI7021        (1<<13) /*       PA13             MOSI      PWM2      22    */
#define SCK7021         (1<<14) /*       PA14             SPCK      PWM3      21    */
#define SSC_TF          (1<<15) /*       PA15             TF        TIOA1     20    */
#define SSC_TK          (1<<16) /*       PA16             TK        TIOB1     19    */
#define SSC_TD          (1<<17) /*       PA17             TD        PCK1       9    */
#define SSC_RD          (1<<18) /*       PA18             RD        PCK2      10    */
#define RXCLK_7021      (1<<19) /*       PA19             RK        FIQ       13    */
#define TXDATA_7021     (1<<19) /*       PA19             RK        FIQ       13    */
#define SSC_RF          (1<<20) /*       PA20             RF        IRQ0      16    */
#define DNSW_           (1<<21) /*       PA21             RXD1      PCK1      11    */
#define UPSW_           (1<<22) /*       PA22             TXD1      NPCS3     14    */
#define LCD_RS          (1<<23) /*       PA23             SCK1      PWM0      15    */
#define LCD_D7          (1<<24) /*       PA24             RTS1      PWM1      23    */
#define LCD_D6          (1<<25) /*       PA25             CTS1      PWM2      25    */
#define LCD_D5          (1<<26) /*       PA26             DCD1      TIOA2     26    */
#define LCD_D4          (1<<27) /*       PA27             DTR1      TIOB2     37    */
#define LCD_E           (1<<28) /*       PA28             DSR1      TCLK1     38    */
#define LCD_BK          (1<<29) /*       PA29             RI1       TCLK2     41    */
#define SWD             (1<<30) /*       PA30             IRQ1      NPCS2     42    */
#define RXDATA_7021     (1<<31) /*       PA31             NPCS1     PCK2      52    */

#define LCD_DATAMASK (0xF<<24)
#define ROTORY_SW_MASK (DNSW_|UPSW_)

/*------------------*/
/* USART Definition */
/*------------------*/
#define UART_BAUD	   115200   // Baud rate

/*--------------*/
/* Master Clock */
/*--------------*/
#define EXT_OC          16384000   // External oscillator MAINCK
#define MCK             49152000   // MCK = [ (EXT_OC/5) * 30 ] / 2 ( see CStartup_SAM7.c)
#define MCKKHz          (MCK/1000) //

#endif /* Board_h */
