/* ==========================================================================*/
/* - - - - - - - - - - -  R F C o n t r o l . H   - - - - - - - - - - - - - -*/
/* ==========================================================================*/
/*    Definition File for the RFControl.c module                             */
/*  Created   2007-03-10	msw                                              */
/*  Modified  2007-03-16    msw                                              */
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
#ifndef RFCTRL_H_
#define RFCTRL_H_
#include "Common.h"

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"

/*---------------------------------------------------------------------------*/
/*------------------> G L O B A L   D E F I N E S <--------------------------*/
/*---------------------------------------------------------------------------*/
#define RFTQ_SIZE 32		//must be power of 2 for wrap around logic
#define RFRQ_SIZE 4		//must be power of 2 for wrap around logic
#define VDATAFRAME_SIZE 3
#define FRAMESYNC_SPACING 21

#define TXSTATE_OFF 0
#define TXSTATE_ON 1
#define TXSTATE_FINISHING 2
#define TXSTATE_DONE 3

#define RXSTATE_SEARCH 0	
#define RXSTATE_HDR 1
#define RXSTATE_VOICE 2


/*---------------------------------------------------------------------------*/
/*-------------------------> T y p e d e f s <-------------------------------*/
/*---------------------------------------------------------------------------*/
typedef union
{
	struct
	{
		U8BIT VData[9];
		U8BIT UsrData[3];
	}data __attribute__((__packed__));
	struct
	{
		U8BIT VData[9];
		U8BIT FrameSync[3];
	}sync __attribute__((__packed__));
	struct
	{
		U8BIT VData[9];
		U8BIT EndSync[3];
	}end __attribute__((__packed__));
	U32BIT Buf32[VDATAFRAME_SIZE];
}tDstarVDFrame;


/*---------------------------------------------------------------------------*/
/*------------------> E x p o s e d    F u n c t i o n s <-------------------*/
/*---------------------------------------------------------------------------*/
void RFControlInit( );
void SetRxFrequency(U32BIT Freq);
void SetTxFrequency(U32BIT Freq);
void SetPTT(BOOL On);
void StartTx();
void EndTx();
void StartRx();
void PutDstarVDFrame();
void PutDstarVSFrame();
void GetDstarVDFrame();


/*---------------------------------------------------------------------------*/
/*------------------> E x p o s e d    V a r i a b l e s <-------------------*/
/*---------------------------------------------------------------------------*/
extern BOOL SquelchOpen;
extern S16BIT RxdBm;
extern tDstarVDFrame RFRxDataQueue[RFRQ_SIZE];
extern U32BIT RFTxDataQueue[RFTQ_SIZE];
extern U32BIT RFTxQHead;
extern volatile U32BIT RFTxQTail;
extern volatile U32BIT RFRxQHead;
extern U32BIT RFRxQTail;
extern U32BIT RxState;
extern U32BIT TxState;
extern volatile U32BIT TxShiftWord;
extern volatile U32BIT TxShiftCount;
extern volatile U32BIT RxShiftWord;
extern volatile U32BIT RxShiftCount;
extern volatile U32BIT RxWordCount;
extern BOOL RxQueueEmpty;


extern U32BIT CurrentRxFreq;
extern U32BIT CurrentTxFreq;

extern U32BIT TxSyncFrameCount;
extern tDstarVDFrame FirstSyncFrame;
extern tDstarVDFrame VSFrame;
extern tDstarVDFrame VDFrame;


#endif /*RFCTRL_H_*/
